
CREATE DATABASE IF NOT EXISTS webo;

USE webo;

-- MySQL dump 8.22
--
-- Host: localhost    Database: webo
---------------------------------------------------------
-- Server version	3.23.56-log

--
-- Table structure for table 'alarms'
--

CREATE TABLE alarms (
  oid int(10) unsigned NOT NULL auto_increment,
  relatedobjectclass varchar(30) default NULL,
  relatedobjectoid int(11) default NULL,
  owner varchar(20) NOT NULL default '',
  summary varchar(60) NOT NULL default '',
  dttrigger datetime NOT NULL default '0000-00-00 00:00:00',
  dtstart datetime NOT NULL default '0000-00-00 00:00:00',
  dtend datetime default NULL,
  timeoffset int(11) NOT NULL default '0',
  waittime int(11) NOT NULL default '300',
  status varchar(20) default 'ACTIVE',
  PRIMARY KEY  (oid),
  KEY relatedObjectClass (relatedobjectclass,relatedobjectoid),
  KEY dttrigger (dttrigger)
) TYPE=MyISAM;

--
-- Table structure for table 'attendees'
--

CREATE TABLE attendees (
  oid int(11) NOT NULL auto_increment,
  event_oid int(11) NOT NULL default '0',
  type enum('USER','CONTACT','EXTERNAL','RESOURCE') NOT NULL default 'USER',
  status enum('NEEDS-ACTION','ACCEPTED','DELEGATED','DECLINED') default NULL,
  person_oid int(11) default NULL,
  name varchar(60) NOT NULL default '',
  email varchar(40) default NULL,
  role enum('ATTENDEE','ORGANIZER','NON-PARTICIPANT') default NULL,
  icon varchar(50) NOT NULL default 'obj/attendee-needsaction',
  bCal tinyint(4) default NULL,
  bMail tinyint(4) default NULL,
  PRIMARY KEY  (oid),
  KEY event_oid (event_oid)
) TYPE=MyISAM;

--
-- Table structure for table 'automatedtasks'
--

CREATE TABLE automatedtasks (
  auto_id int(11) NOT NULL auto_increment,
  auto_type varchar(40) NOT NULL default '',
  auto_user varchar(12) default NULL,
  auto_dtstart datetime default NULL,
  auto_dtend datetime default NULL,
  auto_authkey varchar(32) default NULL,
  auto_run char(1) NOT NULL default 'O',
  PRIMARY KEY  (auto_id)
) TYPE=MyISAM;

--
-- Table structure for table 'autpermission'
--

CREATE TABLE autpermission (
  userid varchar(20) NOT NULL default '',
  role varchar(20) NOT NULL default '',
  PRIMARY KEY  (userid,role)
) TYPE=MyISAM;

--
-- Table structure for table 'autrole'
--

CREATE TABLE autrole (
  role varchar(20) NOT NULL default '',
  status int(11) default '0',
  owner varchar(20) default NULL,
  created datetime default '0000-00-00 00:00:00',
  modified datetime default NULL,
  description varchar(128) default NULL,
  PRIMARY KEY  (role)
) TYPE=MyISAM;

--
-- Table structure for table 'categories'
--

CREATE TABLE categories (
  cat_id int(10) unsigned NOT NULL auto_increment,
  lang char(2) NOT NULL default 'EN',
  label varchar(32) default NULL,
  PRIMARY KEY  (cat_id,lang)
) TYPE=MyISAM;

--
-- Table structure for table 'contact_address'
--

CREATE TABLE contact_address (
  contact_oid int(11) NOT NULL default '0',
  adrtype int(4) NOT NULL default '1',
  pobox varchar(10) default NULL,
  street varchar(30) default NULL,
  extended varchar(30) default NULL,
  postalcode varchar(10) default NULL,
  locality varchar(30) default NULL,
  region varchar(30) default NULL,
  country varchar(30) default NULL,
  KEY contact_oid (contact_oid,adrtype)
) TYPE=MyISAM;

--
-- Table structure for table 'contacts'
--

CREATE TABLE contacts (
  oid int(10) unsigned NOT NULL auto_increment,
  class varchar(20) NOT NULL default 'contact',
  userid varchar(20) default NULL,
  owner varchar(20) NOT NULL default '',
  group_oid int(11) default NULL,
  group_access varchar(5) default NULL,
  categories varchar(64) default NULL,
  name varchar(60) NOT NULL default '',
  firstname varchar(30) default NULL,
  lastname varchar(30) default NULL,
  middlename varchar(30) default NULL,
  prefix varchar(5) default NULL,
  organization varchar(30) default NULL,
  department varchar(30) default NULL,
  title varchar(30) default NULL,
  email1 varchar(40) default NULL,
  email2 varchar(40) default NULL,
  email3 varchar(40) default NULL,
  pagerid varchar(30) default NULL,
  pagerprotocol varchar(20) default NULL,
  phone1 varchar(20) default NULL,
  phone2 varchar(20) default NULL,
  phone3 varchar(20) default NULL,
  phone4 varchar(20) default NULL,
  fax1 varchar(20) default NULL,
  fax2 varchar(20) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  url1 varchar(40) default NULL,
  url2 varchar(40) default NULL,
  initials varchar(5) default NULL,
  publickey mediumtext,
  ical_uid varchar(60) default NULL,
  note text,
  PRIMARY KEY  (oid),
  KEY userid (userid),
  KEY class (class),
  KEY group_oid (group_oid),
  KEY owner (owner)
) TYPE=MyISAM;

--
-- Table structure for table 'customfields'
--

CREATE TABLE customfields (
  owner varchar(20) NOT NULL default '',
  position int(11) NOT NULL default '0',
  name varchar(60) NOT NULL default '',
  value varchar(100) default NULL,
  dtmodified datetime default NULL,
  KEY owner (owner),
  KEY name (name)
) TYPE=MyISAM;

--
-- Table structure for table 'events'
--

CREATE TABLE events (
  oid int(10) unsigned NOT NULL auto_increment,
  owner varchar(20) NOT NULL default '',
  summary varchar(60) NOT NULL default '',
  dtstart datetime default NULL,
  dtend datetime default NULL,
  priority tinyint(4) default NULL,
  transparency tinyint(4) NOT NULL default '0',
  sequence smallint(6) NOT NULL default '0',
  location varchar(60) default NULL,
  privacy enum('PRIVATE','PUBLIC','CONFIDENTIAL') default NULL,
  status enum('TENTATIVE','CONFIRMED','CANCELLED') default NULL,
  recurrule varchar(60) default NULL,
  description varchar(255) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  allday tinyint(4) NOT NULL default '0',
  categories varchar(100) default NULL,
  PRIMARY KEY  (oid),
  KEY owner (owner),
  KEY dtstart (dtstart),
  KEY dtend (dtend),
  FULLTEXT KEY category (categories)
) TYPE=MyISAM;

--
-- Table structure for table 'events_recurrences'
--

CREATE TABLE events_recurrences (
  event_oid int(10) unsigned NOT NULL auto_increment,
  frequency enum('bysecond','byminute','byhour','byday','bymonthday','byyearday','byweekno','bymonth','bysetpos') NOT NULL default 'byday',
  itv int(10) unsigned NOT NULL default '1',
  iterations int(10) NOT NULL default '1',
  range_start datetime default NULL,
  range_end datetime default NULL,
  PRIMARY KEY  (event_oid),
  KEY k_range_start (range_start),
  KEY k_range_end (range_end)
) TYPE=MyISAM;

--
-- Table structure for table 'groupmembers'
--

CREATE TABLE groupmembers (
  group_oid int(11) NOT NULL default '0',
  contact_oid int(11) NOT NULL default '0',
  role varchar(20) default 'member',
  name varchar(60) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  PRIMARY KEY  (group_oid,contact_oid),
  KEY idx_name (name)
) TYPE=MyISAM;

--
-- Table structure for table 'groups'
--

CREATE TABLE groups (
  oid int(10) unsigned NOT NULL auto_increment,
  name varchar(30) NOT NULL default '',
  description varchar(255) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  email varchar(30) default NULL,
  PRIMARY KEY  (oid),
  UNIQUE KEY name (name)
) TYPE=MyISAM;

--
-- Table structure for table 'notes'
--

CREATE TABLE notes (
  oid int(10) unsigned NOT NULL auto_increment,
  owner varchar(20) NOT NULL default '',
  summary varchar(60) NOT NULL default '',
  created datetime default NULL,
  modified datetime default NULL,
  categories varchar(100) default NULL,
  description text,
  ical_uid varchar(64) default NULL,
  PRIMARY KEY  (oid)
) TYPE=MyISAM;

--
-- Table structure for table 'object_flow'
--

CREATE TABLE object_flow (
  objectclass varchar(50) NOT NULL default '',
  oid int(11) NOT NULL default '0',
  status enum('normal','temporary','draft','deleted','incoming','accepted','declined','in-progress','completed') default 'normal',
  owner varchar(20) NOT NULL default '',
  sender varchar(20) default NULL,
  recipient varchar(20) default NULL,
  dtcreated datetime default NULL,
  iscurrent int(2) default '1'
) TYPE=MyISAM;

--
-- Table structure for table 'pagelets'
--

CREATE TABLE pagelets (
  path varchar(80) NOT NULL default '',
  name varchar(60) NOT NULL default '',
  module varchar(30) NOT NULL default '',
  description varchar(255) default NULL,
  width int(11) NOT NULL default '0',
  PRIMARY KEY  (path)
) TYPE=MyISAM COMMENT='pagelets (piece of page) repository';

--
-- Table structure for table 'resources'
--

CREATE TABLE resources (
  oid int(10) unsigned NOT NULL auto_increment,
  name varchar(60) default NULL,
  class varchar(20) NOT NULL default 'room',
  location varchar(30) default NULL,
  description varchar(255) default NULL,
  phone varchar(20) default NULL,
  url varchar(40) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  PRIMARY KEY  (oid),
  UNIQUE KEY name (name)
) TYPE=MyISAM;

--
-- Table structure for table 'sharing'
--

CREATE TABLE sharing (
  objectclass varchar(32) NOT NULL default 'contact',
  objectoid int(11) NOT NULL default '0',
  permissions int(11) NOT NULL default '1',
  group_oid int(11) NOT NULL default '0',
  PRIMARY KEY  (objectclass,objectoid,group_oid)
) TYPE=MyISAM COMMENT='object sharing permissions';

--
-- Table structure for table 'todos'
--

CREATE TABLE todos (
  oid int(10) unsigned NOT NULL auto_increment,
  owner varchar(20) NOT NULL default '',
  assignee varchar(20) default NULL,
  summary varchar(60) NOT NULL default '',
  priority tinyint(4) default NULL,
  percent tinyint(4) default '0',
  privacy enum('PRIVATE','PUBLIC','CONFIDENTIAL') default NULL,
  status enum('NEEDS-ACTION','COMPLETED','IN-PROCESS','CANCELLED') default NULL,
  categories varchar(100) default NULL,
  sequence smallint(6) NOT NULL default '0',
  description text,
  ical_uid varchar(64) default NULL,
  tsduration int(11) default NULL,
  tscreated int(11) default NULL,
  tsstarted int(11) default NULL,
  tsclosed int(11) default NULL,
  tsmodified int(11) default NULL,
  tsdue int(11) default NULL,
  PRIMARY KEY  (oid),
  KEY idx_todos_status (status),
  KEY idx_todos_owner (owner),
  KEY idx_todo_assignee (assignee)
) TYPE=MyISAM;

--
-- Table structure for table 'userprofile'
--

CREATE TABLE userprofile (
  userid varchar(20) NOT NULL default '',
  name varchar(60) NOT NULL default '',
  value varchar(100) default NULL,
  dtmodified datetime default NULL,
  PRIMARY KEY  (userid,name)
) TYPE=MyISAM;

--
-- Table structure for table 'users'
--

CREATE TABLE users (
  userid varchar(20) NOT NULL default '',
  userpassword varchar(40) NOT NULL default '',
  status enum('enabled','disabled','locked','deleted') default NULL,
  lastaccess datetime default NULL,
  PRIMARY KEY  (userid),
  UNIQUE KEY userid (userid)
) TYPE=MyISAM;

--
-- Table structure for table 'vocabulary'
--

CREATE TABLE vocabulary (
  lang varchar(5) NOT NULL default 'en',
  name varchar(255) NOT NULL default '',
  value varchar(255) default NULL,
  PRIMARY KEY  (lang,name)
) TYPE=MyISAM;

--
-- Table structure for table 'webcache'
--

CREATE TABLE webcache (
  wbc_id int(11) NOT NULL auto_increment,
  wbc_url varchar(255) NOT NULL default '',
  wbc_name varchar(60) default NULL,
  wbc_contenttype varchar(50) NOT NULL default '''text/html''',
  wbc_owner varchar(20) default NULL,
  wbc_nextupdate int(11) default NULL,
  wbc_lastupdated int(11) default NULL,
  wbc_cachefile varchar(100) default NULL,
  wbc_status varchar(10) NOT NULL default '''NEEDS-UPD',
  PRIMARY KEY  (wbc_id),
  KEY wbc_url (wbc_url,wbc_owner,wbc_status)
) TYPE=MyISAM;

--
-- Table structure for table 'webofolder'
--

CREATE TABLE webofolder (
  oid int(10) unsigned NOT NULL auto_increment,
  owner varchar(20) NOT NULL default '',
  sharing varchar(30) default NULL,
  class enum('standard','smart','home','abook','recent','trash','document') NOT NULL default 'standard',
  privacy enum('PRIVATE','PUBLIC','SHARED') NOT NULL default 'PRIVATE',
  name varchar(60) NOT NULL default '',
  category varchar(60) default NULL,
  criteria varchar(100) default NULL,
  ordering varchar(50) default NULL,
  created datetime default NULL,
  modified datetime default NULL,
  icon varchar(30) NOT NULL default 'obj/folder',
  diskpath varchar(255) default NULL,
  PRIMARY KEY  (oid),
  KEY idxsharing (sharing),
  KEY idxcategory (category)
) TYPE=MyISAM;

--
-- Table structure for table 'webofoldercontent'
--

CREATE TABLE webofoldercontent (
  parent_oid int(10) unsigned NOT NULL default '0',
  contentClass varchar(40) NOT NULL default 'webofolder',
  child_oid int(10) unsigned NOT NULL default '0',
  name varchar(60) NOT NULL default '',
  icon varchar(30) NOT NULL default 'obj/folder',
  PRIMARY KEY  (parent_oid,contentClass,child_oid)
) TYPE=MyISAM;

--
-- Table structure for table 'webolog'
--

CREATE TABLE webolog (
  logid bigint(20) NOT NULL auto_increment,
  userid varchar(20) default NULL,
  level varchar(8) NOT NULL default 'notice',
  appcode varchar(40) default NULL,
  ts timestamp(14) NOT NULL,
  msg varchar(255) default NULL,
  PRIMARY KEY  (logid),
  KEY userid (userid,level)
) TYPE=MyISAM;

--
-- Table structure for table 'webomessage'
--

CREATE TABLE webomessage (
  oid int(11) NOT NULL auto_increment,
  fromuser varchar(40) NOT NULL default '',
  destuser varchar(20) NOT NULL default '',
  subject varchar(80) default NULL,
  message text,
  relatedObject varchar(30) default NULL,
  status varchar(20) default 'unread',
  icon varchar(20) default 'obj/info',
  created datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (oid)
) TYPE=MyISAM;

-- MySQL dump 8.22
--
-- Host: localhost    Database: webo
---------------------------------------------------------
-- Server version	3.23.56-log

--
-- Dumping data for table 'pagelets'
--


INSERT INTO pagelets VALUES ('calendar/pgl_month','Month calendar','calendar','A small month view of your calendar',120);
INSERT INTO pagelets VALUES ('calendar/pgl_dayoverview','Daily view of events and todos','calendar','Day overview of your meetings and tasks',150);
INSERT INTO pagelets VALUES ('calendar/pgl_list','Meetings list','',NULL,300);
INSERT INTO pagelets VALUES ('tasks/pgl_list','Todo list','tasks','A preview of your Todo List',300);
INSERT INTO pagelets VALUES ('tasks/pgl_addtodo','Todo add form','tasks','A form to quickly add a Todo',300);
INSERT INTO pagelets VALUES ('help/pgl_tip','Help Tips','help','a \"Tip of the day\" pagelet',200);
INSERT INTO pagelets VALUES ('abook/pgl_list','Contacts list','abook','List overview of your Address Book',300);
INSERT INTO pagelets VALUES ('abook/contact/pgl_addcontact','Contact Express Add','abook','Add a contact in your Address book without leaving the Desktop page',200);
INSERT INTO pagelets VALUES ('webspider/pgl_rssfeed','Live news feed','webspider','Live updated Newsline from the Web',300);
INSERT INTO pagelets VALUES ('base/pgl_eval','Expression eval','base','A debug tool that displays variables, mainly for Webo development usage',400);
INSERT INTO pagelets VALUES ('calendar/pgl_pendingevents','Pending meetings','calendar','A list of pending meetings waiting for your reply',300);
INSERT INTO pagelets VALUES ('base/pgl_globalsearch','Global search','base','Search Webo for events,todos,contacts...',400);
INSERT INTO pagelets VALUES ('calendar/event/pgl_addevent','Express Event add','calendar','Pagelet to add a simple event in your calendar',200);
INSERT INTO pagelets VALUES ('calendar/pgl_day','day calendar view','calendar','A small day view',120);

--
-- Dumping data for table 'categories'
--



--
-- Dumping data for table 'vocabulary'
--



--
-- Dumping data for table 'autrole'
--


INSERT INTO autrole VALUES ('webo-user',0,'webo-adm',NULL,NULL,'Grant access to WEBO core functionnalities : Calendar,Addressbook, Tasks, Notes');
INSERT INTO autrole VALUES ('webo-admin',0,'webo-adm',NULL,NULL,'Webo administration module : users, groups');
INSERT INTO autrole VALUES ('directory-admin',1,'webo-adm',NULL,NULL,'Directory managers : not used yet');
INSERT INTO autrole VALUES ('webo-config',0,'webo-adm','0000-00-00 00:00:00','0000-00-00 00:00:00','Allow user to configure WEBO (configuration pages)');
INSERT INTO autrole VALUES ('webo-developer',0,'webo-adm','0000-00-00 00:00:00','0000-00-00 00:00:00','Grant access to Webo developpement and debug tools');
INSERT INTO autrole VALUES ('anonymous',0,'webo-adm',NULL,NULL,'Dummy role for anonymous access to some Webo pages');

#
# default admin user and Permission
#
INSERT INTO users (userid, userpassword, status, lastaccess) VALUES ('webo-adm', MD5('webo-adm'), 'enabled', NULL);
INSERT INTO contacts (oid, class, userid, owner, group_oid, group_access, categories, name, firstname, lastname, middlename, prefix, organization, department, title, email1, email2, email3, pagerid, pagerprotocol, phone1, phone2, phone3, phone4, fax1, fax2, created, modified, url1, url2, initials, publickey, ical_uid, note) VALUES ('1', 'user', 'webo-adm', 'webo-adm', NULL, NULL, NULL, 'Webo Administrator', NULL, NULL, NULL, NULL, NULL, NULL, NULL, '%weboadmin_email%', NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, '', '', NULL, NULL, NULL, NULL, NULL, NULL);
# TODO : profile

# grant all roles to webo-adm
INSERT INTO autpermission SELECT 'webo-adm', role FROM autrole WHERE role<>'anonymous';

# SAMPLE event
INSERT INTO `events` (`oid`, `owner`, `summary`, `dtstart`, `dtend`, `priority`, `transparency`, `sequence`, `location`, `privacy`, `status`, `recurrule`, `description`, `created`, `modified`, `allday`, `categories`)
   VALUES ('1', 'webo-adm', 'Webo installed', NOW(), NOW(), '5', '1', '1', NULL, 'PUBLIC', 'CONFIRMED', NULL, NULL, NOW(), NOW(), '1', NULL);

-- default 'userprofile'
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsLeft[0]','calendar/pgl_month',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsLeft[1]','calendar/pgl_dayoverview',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsLeft[2]','help/pgl_tip',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsLeft[3]','calendar/event/pgl_addevent',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsLeft[4]','abook/contact/pgl_addcontact',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsRight[0]','calendar/pgl_pendingevents',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsRight[1]','tasks/pgl_list',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsRight[2]','abook/pgl_list',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsRight[3]','webspider/pgl_rssfeed',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_pageletsRight[4]','tasks/pgl_addtodo',NULL);
INSERT INTO userprofile VALUES ('(default)','abook_cardview_size','3-3',NULL);
INSERT INTO userprofile VALUES ('(default)','abook_listview_size','25',NULL);
INSERT INTO userprofile VALUES ('(default)','abook_pagesize','25',NULL);
INSERT INTO userprofile VALUES ('(default)','alarm_byemail','1',NULL);
INSERT INTO userprofile VALUES ('(default)','alarm_desktop','1',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_endtime','18',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_eventduration','3600',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_extraeventlist','computer',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_holidays','uk',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_rescheduleoffset','1800',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_starttime','9',NULL);
INSERT INTO userprofile VALUES ('(default)','calendar_weekview_with_todos','1',NULL);
INSERT INTO userprofile VALUES ('(default)','defaultSharing',NULL,NULL);
INSERT INTO userprofile VALUES ('(default)','defaultSharingAccess','RW',NULL);
INSERT INTO userprofile VALUES ('(default)','defaultSharingGroup',NULL,NULL);
INSERT INTO userprofile VALUES ('(default)','lang','uk',NULL);
INSERT INTO userprofile VALUES ('(default)','locale','en',NULL);
INSERT INTO userprofile VALUES ('(default)','tasks_assignee',NULL,NULL);
INSERT INTO userprofile VALUES ('(default)','tasks_listview_size','25',NULL);
INSERT INTO userprofile VALUES ('(default)','theme_fontFamily','Arial,Helvetica,Verdana',NULL);
INSERT INTO userprofile VALUES ('(default)','theme_fontSize','10px',NULL);
INSERT INTO userprofile VALUES ('(default)','theme_paneltitlebg',NULL,NULL);
INSERT INTO userprofile VALUES ('(default)','theme_theme','blue',NULL);
INSERT INTO userprofile VALUES ('(default)','theme_wallpaper',NULL,NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[0]','task/about',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[1]','obj/contacts',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[2]','obj/tasks',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[3]','task/newevent',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[4]','task/newtodo',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[5]','task/newcontact',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[6]','task/configure',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[7]','task/print',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[8]','task/help',NULL);
INSERT INTO userprofile VALUES ('(default)','desktop_shortcuts[9]','task/logout',NULL);

# add a profile for webo-adm
CREATE TABLE tmpup AS SELECT * FROM userprofile WHERE userid='(default)';
INSERT INTO userprofile SELECT 'webo-adm',name,value,NOW() FROM tmpup WHERE userid='(default)';
DROP TABLE tmpup;

