<?php/*
	Utility class
	try to guess a date in any format
	guesses are:
	a)  hier => yesterday @ 0h
  	b)  12 => the 12 of current month @ 0h
	c)  auj, auj., aujour[any car] => now (today @ current time)
	d)  1/3 => the 1 march of this year if dateOrder is set to DMY ; or the 3th of january if dateOrder is set to MDY
	e)  1/3/01 => 1 march 2001 or 3 january 2001, depending on SmartDate::dateOrder

	use setDefaultTimestamp() to modify the assumed default year and month in guess b), d)
	
	@Usage
		$smart = new SmartDate();
		$smart->parse( "12/5/73" );
		
		// the default year will be 1999
		$smart->setDefaultTimestamp( mktime( 0,0,0,1,1, 1999 );
		$smart->parse( "31/12" ); // => 31 december 1999

*/
class SmartDate
{
	var 
		$dateOrder = "DMY"; // day/mon/year order - change it to MDY for Mon/Day/Year format
		
	function SmartDate()
	{
	}
	
	function setDefaultTimestamp( $epoq )
	{
		$this->defaultDate = getdate( $epoq );
	}
		
	function Parse( $str ) {
		
		$dateSeparator = "[/-]";
		$timeSeparator = "[:hH-]";
		$hourSeparator = "[:hH]";
		$minuteSeparator = "[:mM]";

		$str = trim(strtolower($str));
		$today = getdate(time());
		
			
		// "hier" string
		if( $str == "hier" )	{

			return mktime( 0,0,0, $today['mon'], $today['mday']-1, $today['year'] );

		// misc way of saying "aujourd'hui"
		} 
		elseif( $str == "auj" || $str == "auj." || strpos( "aujour", $str ) === 0 )	{

			return time();

		// number => we consider it as the day in default month/year
		} 
		elseif( preg_match( "|^(\d?\d)$|", $str, $regs ) ) {
		
			if( ! isset($this->defaultDate) )
				$this->defaultDate = $today;
			return mktime( 0,0,0, $this->defaultDate['mon'], $regs[1], $this->defaultDate['year'] );
		
		// detect a date
		} 
		elseif( preg_match( "|(\d+)$dateSeparator(\d+)($dateSeparator(\d+))?(.*)|", $str, $regs ) ) {

			if( $this->dateOrder == "DMY" ) {
				$D = $regs[1];
				$M = $regs[2];
			} else {
				$M = $regs[1];
				$D = $regs[2];
			}
			$Y = $regs[4];
			
			if( intval($Y) ) {
				// handle 1 or 2 digits years
				if( $Y < 100 )
					$Y = 2000 + $Y;
				
			} else {
				
				$Y = $this->defaultDate['year'];
			}
				
			// to avoid time confusion with date
			$str = $regs[count($regs)-1];

		// defaults
		} 
		elseif( isset( $this->defaultDate ) ) {
			
			$Y = $this->defaultDate['year'];
			$M = $this->defaultDate['mon'];
			$D = $this->defaultDate['mday'];
			
		}
		else {
		
			return -1;
		}
				
		// detect a time 
		if( preg_match( "|(\d+)$hourSeparator((\d+)$minuteSeparator?)?|", $str, $regs ) ) {
			if( count($regs) == 2 ) {
				$h = $regs[1];
			} else {
				$h = $regs[1];
				$m = $regs[2];
			}

		} else {
		
			$h = $this->defaultDate['hours'];
			$m = $this->defaultDate['minutes'];
		}
		
		$timestamp = mktime( $h, $m, 0, $M, $D, $Y );

//		echo "D:$D M:$M Y:$Y h:$h m:$m s:$s => $timestamp =>", strftime( "%Y %m %d T %H:%M:%S", $timestamp );

		return $timestamp;		
	}
	
}
