<?php
//!! DB
/*!

        Database library is a portable database access layer, MySQL version

        offers a set of methods to access a single database both on low level (connect, query)
        and higher level. provides some method to load, insert and update hash array and objects
        in a one line piece of code !


\code

        db_connect("mysql://root:password@dbserver/webo");

        inserting an object $user. column names must match User class fieldnames
        fieldnames "_" prefixed will be ignored, (considered as non persistents)

        db_insertobject( "T_USERS", $user );

        inserting an object $user, in a table with an autoincrement ID. this id will be returned in $user->oid
        db_insertobject( "T_USERS", &$user, "oid" );

        updating a user. "oid" will be used for the pivot field : "UPDATE...WHERE oid=$user->oid"
        db_updateobject( "T_USERS", $user, "oid" );

\endcode

HISTORY

        $Log: db.mysql.php,v $
        Revision 1.1  2003/05/18 07:19:45  lwest
        renamed db layer file

        Revision 1.1  2003/05/09 21:10:46  lwest
        DB directory moved to db

        Revision 1.14  2003/05/05 10:02:01  lwest
        added method db_close

        Revision 1.13  2003/04/12 23:53:01  lwest
        *** empty log message ***

        Revision 1.12  2003/04/12 17:32:37  lwest
        changes in webo::log prototype

        Revision 1.11  2003/04/08 03:15:22  lwest
        removed commented code

        Revision 1.10  2003/04/02 13:14:33  lwest
        *** empty log message ***

        Revision 1.9  2003/03/24 00:50:44  lwest
        LOG_WARN problem fix

        Revision 1.8  2003/03/21 22:28:02  lwest
        *** empty log message ***

        Revision 1.7  2003/03/20 01:24:04  lwest
        fixed bugs

        Revision 1.6  2003/03/18 01:26:39  lwest
        removed bad icons


*/


if( ! extension_loaded('mysql')) {
        dl( 'mysql.so' );
}


function db_connect( $url )
{
        $db = parse_url( $url );

        ( $db['scheme'] == 'mysql' ) or die( "Database not supported: $dbinfos[scheme]" );

        isset( $db["host"]) or die( "db_connect : host undefined" );
        isset( $db["port"]) or $db["port"] = 3306;
        isset( $db["user"]) or $db["user"] = NULL;
        isset( $db["pass"]) or $db["pass"] = "";

        if( isset($db['path']))
                $db_name = substr( $db['path'], 1 );
        else
                $db_name = "";

        $DB_cnx = mysql_pconnect( "$db[host]:$db[port]", $db['user'], $db['pass'] ) or die( 'Database connexion failed - See install file' );

        if( $db_name )
                mysql_select_db( $db_name ) or die( "Database not found ($db[name])" );

        return true;

}


/* DB Layer interface methods below */

function db_error( $errormsg = NULL )
{
   if( $errormsg == NULL )
      $errormsg = "Error in lib.mysql.php [" . mysql_errno() . "] " . mysql_error();
   Webo::log( $errormsg, LOG_WARNING );
   return false;
}

function db_errno()
{
        return mysql_errno();
}

function db_loadobject( $sql, &$object )
{
   $hash = array();
   if( ! db_loadhash( $sql, &$hash ) ) {
      db_error();
      return false;
   }
   foreach( $hash as $k=>$v )
   {
      $object->$k = $v;
   }
   return true;
}

function db_exec( $sql )
{
        $cur = mysql_query( $sql );
        if( ! $cur ) {
                return false;
        }
        return $cur;
}


function db_prepare( $sqlTemplate, $debug=0 )
{
   $sql = $sqlTemplate;
   $sth = new stdclass();
   $sth->template = $sqlTemplate;
   $sth->argcount = 0;
   $sth->sqlparts = array();
   $lastpos = 0;
   while( ($pos = strpos($sql, "?", $lastpos)) != false ) {
            $substr = substr( $sql, $lastpos,$pos-$lastpos);
            $sth->sqlparts[] = $substr;
            $lastpos = $pos+1;
   }
   $sth->sqlparts[] = substr( $sql, $lastpos); // last part of sql
   $sth->argcount = count($sth->sqlparts) -1;
   if( $debug ) {
      echo "<pre>"; print_r($sth); echo "</pre>";
   }
   return $sth;
}

function db_execute()
{
   $sth = func_get_arg(0);
   $cnt = func_num_args() -1;
   if( $sth->argcount != $cnt ) {
      return db_error( "db_execute() : wrong number of parameters  for ". $sth->template . " - $sth->argcount parameters expected" );
   }
   $sql = "";
   for( $i=0; $i< $sth->argcount; $i++ )
   {
      $param = func_get_arg($i+1);
      $sqlp = $sth->sqlparts[$i];
      // TODO : manage  arguments without quotes
      $sql .= $sqlp . "'" . db_escape($param) ."'";
   }
   $sql .= $sth->sqlparts[$i];
   $sth->sql = $sql;
   $cur = mysql_query( $sth->sql );
   if( ! $cur )
      return db_error();
   // print_r($sth);
   return $cur;
}

function db_query( $sql )
{
   $cur = mysql_query( $sql );
   if( ! $cur ) {
      return DB_error();
   }
   $count = mysql_affected_rows();
   return $count;
}

function db_nextrow( $cur )
{
   if( $cur == false )
      return NULL;
   $ret = mysql_fetch_assoc($cur);
   return $ret;
}

function db_loadhash( $sql, &$hash )
{
   $cur = mysql_query( $sql );
   if( ! $cur )
      return db_error();
   $hash = mysql_fetch_assoc( $cur );
   mysql_free_result($cur );
   if( $hash == false  )
      return false;
   else
      return true;
}

function db_loadhashList( $sql )
{
   $cur = mysql_query( $sql );
   if( ! $cur )
      return db_error();
   $hashlist = array();
   while( $hash = mysql_fetch_array( $cur, MYSQL_NUM ) ) {
      $hashlist[$hash[0]] = $hash[1];
   }
   mysql_free_result($cur);
   return $hashlist;
}

function db_loadarray( $sql, $maxrows = NULL )
{
   $cur = mysql_query( $sql );
   if( ! $cur )
      return db_error();
   $array = array();
   while( $hash = mysql_fetch_array( $cur, MYSQL_NUM ) ) {
      $array[] = $hash[0];
   }
   mysql_free_result($cur);
   return $array;
}


function db_loadlist( $sql, $maxrows = NULL )
{
   $cur = mysql_query( $sql );
   $cur or db_error();
   $list = array();
   $cnt = 0;
   while( $hash = mysql_fetch_array( $cur, MYSQL_ASSOC ) ) {
      $list[] = $hash;
      if( $maxrows && $maxrows == $cnt++ )
         break;
   }
   mysql_free_result($cur);
   return $list;
}

/*
        @note to optimize request, just SELECT object's oid in $sql
*/
function db_loadobjectList( $sql, $object, $maxrows = NULL )
{
        $cur = mysql_query( $sql );
        if( ! $cur ) die( "db_loadobjectList : " . db_error() );
        $list = array();
        $cnt = 0;
        while( $row = mysql_fetch_array( $cur ) ) 
        {
                $list[] = $object->Load( $row[0] );
                if( $maxrows && $maxrows == $cnt++ )
                        break;
        }
        mysql_free_result($cur);
        return $list;
}

function db_inserthash( $table, &$hash, $keyName = NULL )
{
        $fmtsql = "insert into $table ( %s ) values( %s ) ";
        foreach( $hash as $k => $v ) {
                if( is_array($v) or is_object($v) or $v == NULL )
                        continue;
                $fields[] = $k;
                $values[] = "'" . mysql_escape_string( $v ) . "'";
        }
        $sql = sprintf( $fmtsql, implode( ",", $fields ) ,  implode( ",", $values ) );
        if( ! mysql_query( $sql ) )
            return db_error();
        $id = mysql_insert_id();
        if( $keyName && $id )
                $hash[$keyName] = $id;
        return true;
}

function db_insertobject( $table, &$object, $keyName = NULL )
{
        $fmtsql = "insert into $table ( %s ) values( %s ) ";
        foreach( get_object_vars( $object ) as $k => $v ) {
                if( is_array($v) or is_object($v) or $v == NULL )
                        continue;
                if( $k[0] == '_' ) // internal field
                        continue;
                $fields[] = $k;
                $values[] = "'" . mysql_escape_string( $v ) . "'";
        }
        $sql = sprintf( $fmtsql, implode( ",", $fields ) ,  implode( ",", $values ) );
        if( ! mysql_query( $sql ) )
                return db_error();
        $id = mysql_insert_id();
        if( $keyName && $id )
                $object->$keyName = $id;
        return true;

}

function db_updatehash( $table, &$hash, $keyName )
{
        if( ! $keyName ) die( "keyName must be defined in db_updatehash" );

        $fmtsql = "UPDATE $table SET %s WHERE %s";
        foreach( $hash as $k => $v ) {

                if( is_array($v) or is_object($v) or $k[0] == '_' ) // internal or NA field
                        continue;

                if( $k == $keyName ) { // PK not to be updated
                        $where = "$keyName='" . mysql_escape_string( $v ) . "'";
                        continue;
                }
                if( $v == '' )
                        $val = 'NULL';
                else
                        $val = "'" . mysql_escape_string( $v ) . "'";
                $tmp[] = "$k=$val";
        }
        $sql = sprintf( $fmtsql, implode( ",", $tmp ) , $where );
        $ret = mysql_query( $sql );
        return $ret;
}


function db_updateobject( $table, &$object, $keyName )
{
   $fmtsql = "UPDATE $table SET %s WHERE %s";
   foreach( get_object_vars( $object ) as $k => $v )
   {
      if( is_array($v) or is_object($v) or $k[0] == '_' ) // internal or NA field
         continue;

      if( $k == $keyName ) { // PK not to be updated
         $where = "$keyName='" . mysql_escape_string( $v ) . "'";
         continue;
      }
      if( $v == '' )
         $val = 'NULL';
      else
         $val = "'" . mysql_escape_string( $v ) . "'";
      $tmp[] = "$k=$val";
   }
   $sql = sprintf( $fmtsql, implode( ",", $tmp ) , $where );
   $ret = mysql_query( $sql );
   return $ret;
}


/*!
        encrypt a password using the database mecanism if available
        we use the PASSWORD() function for MySQL port
*/
function db_password( $clearpass )
{
   $epass = db_escape( $clearpass );
   $ret = db_loadhash( "SELECT MD5('$epass') as encrypted", $tmp )
      or die( "Password encryption failed" );
   return $tmp['encrypted'];
}


/*!
        format a date+time field according to underlying database format
        @param timestamp : a unix timestamp (seconds since 1/1/1970)
        @return a DB compliant datetime string , or NULL if param is null
*/
function db_datetime( $timestamp )
{
   if( $timestamp == NULL )
      return NULL;

   if( is_object($timestamp) && get_class($timestamp) == 'date' )
      return $timestamp->toString( '%Y-%m-%d %H:%M:%S');
   else
      return strftime( '%Y-%m-%d %H:%M:%S', $timestamp );
}

/*!
         escape a string for use in a SQL query as a data field
         @param a string or any data
         @return the string escaped
*/
function db_escape( $str )
{
   return mysql_escape_string( $str );
}


function db_close()
{
   return mysql_close();
}



?>
