<?php


class File_CSV_Writer
{
	/****************
	 * ces variables sont *private*
	 ***************/
	
	var $filename; 		// nom du fichier temporaire
	var $valide;		// true si fichier valide, false sinon
	
	/// le descripteur du fichier
	var $fo = NULL; 
	
	/// le car de fin de ligne \n (OS bien gaul) ou \r\n (Win32)
	var $eol = "\r\n";
	
	/// le caractere de separation des champs
	var $separator = ";"; 
	
	/// numero de ligne courant
	var $linenumber = 0;
	 
	/// nombre de champs sur chaque ligne
	var $fieldcount = 0;
	
	
	function File_CSV_Writer( $filename, $separator = ";" )
	{
		$this->filename = $filename;
		$this->separator = $separator;
		$this->transformers = array();
		$this->valide = true;
		$this->linenumber = 0;
		$this->fo = fopen( $this->filename, "w" )
			or die( "can not create file $filename" );
		$this->firstseekpos = ftell($this->fo);
	}
	
	function fieldCount()
	{
		return count($this->fields);
	}

	function writeFieldnames( $callback = NULL )
	{
		$fieldnames = array_values( $this->fields );
		$line = implode( $this->separator, $fieldnames );
		$ret = fputs( $this->fo, $line . $this->eol );
		$this->linenumber ++;
		return $ret;
	}
	
	
	/*!
	 	definit le champ a ecrire dans la colonne numero $column
	 	@param column l'index de la colonne
	 	@param fieldname le nom du champ dans le hash fourni en entree lors de l'appel de write()
	 */
	function addFieldDescriptor( $column, $fieldname )
	{
		$this->fields[$column] = $fieldname;
		$this->fieldcount ++;
	}

	/*!
	 	definit tous les champs d'un coup
	 	@param hash la table de mapping des champs ss la forme index => nom_champ
	 	a noter : la methode addFieldDescriptor peut etre invoque apres celle-ci (pour ajouter des champs)
	 	mais pas l'inverse : setFieldDescriptors "ecrase" tout mapping deja defini
	 */
	function setFieldDescriptors( $fieldsmap )
	{
		$this->fields = $fieldsmap;
		$this->fieldcount = count($fieldsmap);
	}

	/*!
		definit un callback de transformation pour le champ $fieldname
		le callback est une function permettant de modifier au vol un champ avant son ecriture dans le fichier
		il prend unn parametre, la valeur du champ et renvoie la valeur a ecrire ds le fichier
		
		@param le nom du champ a transformer
		@param la fonction callback a invoquer
		
		@example
			function strpad( $data ) { return sprintf( "%04.2f", $data ) };
			$csvwriter->addFieldTransformer( "salaire", "strpad" );
			
	*/
	
	function addFieldTransformer( $fieldname, $cbFunction )
	{
		$this->transformers[$fieldname] = $cbFunction;
	}
	
		
	/***********
	* renvoi true si fin de fichier atteinte
	***********/
	function eof()
	{
		return feof( $this->fo );
	}
	
	
	/*
		ecrit l'enregistrement $hash dans le fichier CSV
		$hash est un hash inddexe par les noms symboliques de colonnes definis 
		prealablement par appel de addFieldDescriptor
		si des callback de transformation ont ete definie, elles sont invoques avant l'ecriture du champ
	*/
	function write( $hash )
	{
		$elems = array();
		
		foreach( $this->fields as $idx => $fieldname ) {
			
			$data = @$hash[$fieldname];
			if( isset($this->transformers[$fieldname]) ) {
				$cb = $this->transformers[$fieldname];
				$data = $cb( $data );
			}
			
			// if( empty($data)) log_debug( "no data for field $fieldname" );
	
			$elems[$idx] = $data;
		}
		$line = implode( $this->separator, $elems );
		$this->linenumber ++;
		fputs( $this->fo, $line );
		fputs( $this->fo, $this->eol );
		return true;
	}
		
	/******************
	* Fermeture du fichier en cours
	******************/
	function close()
	{
		if( $this->fo ) {
			fclose( $this->fo );
			$this->fo = NULL;
		}
	}
	
} // end class File_CSV

?>