<?php

/*


   this class encapsulates the PHP mail() function.
   implements CC, Bcc, Priority headers


@version	1.8

- added Content-ID header for each attachment ( for html and image handling)
- changed Content-type  to multipart/related for mail instead of multipart/mixed
- added ReplyTo( $address ) method
- added Receipt() method - to add a mail receipt
- added optionnal charset parameter to Body() method. this should fix charset problem on some mail clients

@example

   include "libmail.php";

   $m= new Mail; // create the mail
   $m->From( "leo@isp.com" );
   $m->To( "destination@somewhere.fr" );
   $m->Subject( "the subject of the mail" );	

   $message= "Hello world!\nthis is a test of the Mail class\nplease ignore\nThanks.";
   $m->Body( $message);	// set the body
   $m->HTMLBody( $htmlmessage); // set the body for an HTML message
   $m->Cc( "someone@somewhere.fr");
   $m->Bcc( "someoneelse@somewhere.fr");
   $m->Priority(4) ;	// set the priority to Low 
   $m->Attach( "/home/leo/toto.gif", "image/gif" ) ;	// attach a file of type image/gif
   $m->Send();	// send the mail
   echo "the mail below has been sent:<br><pre>", $m->Get(), "</pre>";


LASTMOD
   Fri Oct  6 15:46:12 UTC 2000

@author	Leo West - lwest@free.fr

*/

if ( ! function_exists("mail" ))
   die( "PHP mail() function is required\n" );

class Mail
{
   /*
   list of To addresses
   @var	array
   */
   var $sendto = array();
   /*
   @var	array
   */
   var $acc = array();
   /*
   @var	array
   */
   var $abcc = array();
   /*
   paths of attached files
   @var array
   */
   var $aattach = array();
   /*
   list of message headers
   @var array
   */
   var $xheaders = array();
   /*
   message priorities referential
   @var array
   */
   var $priorities = array( '1 (Highest)', '2 (High)', '3 (Normal)', '4 (Low)', '5 (Lowest)' );
   /*
   character set of message
   @var string
   */
   var $charset = "iso-8159-1";
   var $ctencoding = "8bit";
   /*
   shall sender be notified when user read the mail
   @var string
   */

   var $receipt = 0;

   /*
   text mesage
   @var string
   */
   var $body = "";

/*

   Mail contructor

*/

function Mail()
{
   $this->autoCheck( true );
   $this->boundary= "--" . md5( uniqid("myboundary") );
}


/*!
static method that return a pretty email Address in the format "label <email>"
pretty address can then be used in To(), From() etc...
Note : no email verification is done !
@param email string the mail address
@param label the text, usually the person's name
@return a pretty email like "name<email>" , or a simple email of label is empty
@static
*/
function prettyMailAddress( $email, $label )
{
   if( $label == NULL )
      return $email;
   else
      return sprintf( "\"%s\" <%s>", $label, $email );
}

/*

activate or desactivate the email addresses validator
ex: autoCheck( true ) turn the validator on
by default autoCheck feature is on

@param boolean	$bool set to true to turn on the auto validation
@access public
*/
function autoCheck( $bool )
{
   if( $bool )
      $this->checkAddress = true;
   else
      $this->checkAddress = false;
}


/*

Define the subject line of the email
@param string $subject any monoline string

*/
function Subject( $subject )
{
   $this->xheaders['Subject'] = strtr( $subject, "\r\n" , "  " );
}


/*

set the sender of the mail
@param string $from should be an email address

*/

function From( $from )
{

   if( ! is_string($from) ) {
      echo "Class Mail: error, From is not a string";
      exit;
   }
   $this->xheaders['From'] = $from;
}

/*
set the Reply-to header
@param string $email should be an email address

*/
function ReplyTo( $address )
{

   if( ! is_string($address) )
      return false;

   $this->xheaders["Reply-To"] = $address;

}


/*
add a receipt to the mail ie.  a confirmation is returned to the "From" address (or "ReplyTo" if defined)
when the receiver opens the message.

@warning this functionality is *not* a standard, thus only some mail clients are compliants.

*/

function Receipt()
{
   $this->receipt = 1;
}


/*
set the mail recipient
@param string $to email address, accept both a single address or an array of addresses

*/

function To( $to )
{

   // TODO : test validit sur to
   if( is_array( $to ) )
      $this->sendto= $to;
   else
      $this->sendto[] = $to;

   if( $this->checkAddress == true )
      $this->checkAdresses( $this->sendto );

}


/*		Cc()
*		set the CC headers ( carbon copy )
*		$cc : email address(es), accept both array and string
*/

function Cc( $cc )
{
   if( is_array($cc) )
      $this->acc= $cc;
   else
      $this->acc[]= $cc;

   if( $this->checkAddress == true )
      $this->CheckAdresses( $this->acc );

}



/*		Bcc()
*		set the Bcc headers ( blank carbon copy ).
*		$bcc : email address(es), accept both array and string
*/

function Bcc( $bcc )
{
   if( is_array($bcc) ) {
      $this->abcc = $bcc;
   } else {
      $this->abcc[]= $bcc;
   }

   if( $this->checkAddress == true )
      $this->CheckAdresses( $this->abcc );
}


/*		Body( text [, charset] )
*		set the body (message) of the mail
*		define the charset if the message contains extended characters (accents)
*		default to us-ascii
*		$mail->Body( "ml en franais avec des accents", "iso-8859-1" );
*/
function Body( $body, $charset="" )
{
   $this->body = $body;
   if( $charset != "" ) {
      $this->charset = strtolower($charset);
      $this->xheaders["Content-Type"] = "text/plain; charset=$this->charset";
   }
}

/*
 * $mail->HTMLBody( htmlcode )
 */
function HTMLBody( $htmlmessage, $charset='iso-8859-1')
{
   $this->body = $htmlmessage;
   $this->charset = $charset; // TODO : fix that
   $this->xheaders["Mime-Version"] = "1.0";
   $this->xheaders["Content-Type"] = "text/html; charset=$this->charset";
   $this->xheaders["Content-Transfer-Encoding"] = $this->ctencoding;
}

/*		Organization( $org )
*		set the Organization header
*/

function Organization( $org )
{
   if( trim( $org != "" )  )
      $this->xheaders['Organization'] = $org;
}


/**
* Priority
*  set the mail priority flag
*  @param integer  the priority between 1 (highest priority) and 5 ( lowest )
*  @return boolean true if priority is ok, false otherwise
**/
function Priority( $priority )
{
   if( ! intval( $priority ) )
      return false;

   if( ! isset( $this->priorities[$priority-1]) )
      return false;

   $this->xheaders["X-Priority"] = $this->priorities[$priority-1];

   return true;

}


/**
*  Attach a file to the mail
*  
*  @param string $filename : path of the file to attach
*  @param string $mimetype : MIME-type of the file. default to 'application/x-unknown-content-type'
*  @param string $disposition : instruct the Mailclient to display the file if possible ("inline") or always as a link ("attachment") possible values are "inline", "attachment"
*  @param string $friendlyName : friendly filename as seen by the mail recipients
*
*  @example $mail->Attach( "/tmp/php_upload.tmp", "image/jpeg", "attachment", "image.jpeg" );
**/

function Attach( $filename, $mimetype = "", $disposition = "inline", $friendlyName = "" )
{
   // TODO : si filetype="", alors chercher dans un tablo de MT connus / extension du fichier
   if( $filetype == "" )
      $filetype = "application/x-unknown-content-type";

   if( empty($friendlyName) ) 
      $friendlyName = basename($filename);

   $this->attachments[] = array( 
      "mimeType" => $mimetype,
      "disposition" => $disposition,
      "friendlyName" => $friendlyName,
      "filename" => $filename
   );
}

/**
* Add an attachment to the mail - in contrary to attach, the file content is given in a string
*  @param string $data is the content of the attachment
*  @param string $mimetype : MIME-type of the file. default to 'application/x-unknown-content-type'
*  @param string $disposition : instruct the Mailclient to display the file if possible ("inline") or always as a link ("attachment")
*  @param string $friendlyName : filename as seen by the mail recipients
*
*  @example $mail->Attach( "", "image/jpeg", "attachment", "image.jpeg" );
**/
function AttachString( $data, $mimetype = "", $disposition = "inline", $friendlyName = "file.whatever" )
{
   if( $filetype == "" )
      $filetype = "application/x-unknown-content-type";

   $this->attachments[] = array( 
      "mimeType" => $mimetype,
      "disposition" => $disposition,
      "friendlyName" => $friendlyName,
      "data" => $data
   );

}


/**
* Fornat and send the mail using the php mail() builtin function
**/ 
function Send()
{
   $this->BuildMail();

   $this->strTo = implode( ", ", $this->sendto );

   // envoie du mail
   $res = mail( $this->strTo, $this->xheaders['Subject'], $this->fullBody, $this->headers );
   return $res;
}


/**
* Return the whole e-mail , headers + message
*  can be used for displaying the message in plain text or saving it in a file
**/
function &toString()
{
   $this->BuildMail();
   $mail = "To: " . $this->strTo . "\n";
   $mail .= $this->headers . "\n";
   $mail .= $this->fullBody;
   return $mail;
}


/**
* do the same as toString()
*	 @deprecated you must use toString() instead
**/
function Get()
{
   return $this->toString();
}


/**
* Check an email address validity
*  @access public
*  @param string $address : email address to check
*  @return true if email adress is ok
**/
function ValidEmail($address)
{
   if( preg_match( "/^(.*)\<(.+)\>$/", $address, $regs ) ) {
      $address = $regs[1];
   }
   if( preg_match( "/^[^@  ]+@([a-zA-Z0-9\-.]+)$/",$address) )
      return true;
   else
      return false;
}


/**
* Check the validity of several email addresses
*  @param	$aad array of emails addresses to check
*  if any address is unvalid, output an error message and exit, this behavior may be customized
**/
function CheckAdresses( $aad )
{
   for($i=0;$i< count($aad); $i++ ) {
      if( ! $this->ValidEmail( $aad[$i]) ) {
         Webo::log( "class Mail, method checkAdresses : invalid address $aad[$i]" );
      }
   }
}


/**
* Built attachments as mimeparts internal use only
*  @scope private
**/
function _build_attachement()
{

   $this->xheaders["Content-Type"] = "multipart/mixed;\n boundary=\"$this->boundary\"";

   $ata= array();
   $k=0;

   // for each attached file
   for( $i=0; $i < count( $this->attachments); $i++ ) {

      $atc =& $this->attachments[$i];

      $ctype = $atc['mimeType']; // content-type
      $disposition = $atc['disposition'];
      $friendlyName = strtr( "\"'", '__', $atc['friendlyName'] );

      $subhdr = "--$this->boundary\n";
      $subhdr .= "Content-Type: $ctype;\n  name=\"$friendlyName\"\n";
      $subhdr .= "Content-Transfer-Encoding: base64\n";
      $subhdr .= "Content-ID: <part$i>\n";
      $subhdr .= "Content-Disposition: $disposition;\n  filename=\"$friendlyName\"\n";
      $ata[$k++] = $subhdr;

      // this is a file attachment
      if( isset($atc['filename']) ) {

         if( ! file_exists( $atc['filename'] ) ) {
            echo "Class Mail, method attach : file $atc[filename] can't be found";
            exit(-1);
         }
         $fp = fopen( $filename, 'r' );
         if( ! $fp )
            die( "Error: no such file $filename ($php_errormsg)" );
         $filesize = filesize( $filename);
         $ata[$k++] = chunk_split(base64_encode(fread( $fp, $filesize)));
         fclose($fp);

      // this is a string attachment
      } else {

         $ata[$k++] = chunk_split(base64_encode($atc['data']));

      }
   }
   $ata[$k++] = "--$this->boundary--\n";

   $this->fullBody .= implode( "\n", $ata );
}


/**
* Build the email message
*  @access private
**/
function BuildMail()
{

   // build the headers
   $this->headers = "";
//	$this->xheaders['To'] = implode( ", ", $this->sendto );

   if( count($this->acc) > 0 )
      $this->xheaders['CC'] = implode( ", ", $this->acc );

   if( count($this->abcc) > 0 )
      $this->xheaders['BCC'] = implode( ", ", $this->abcc );


   if( $this->receipt ) {
      if( isset($this->xheaders["Reply-To"] ) )
         $this->xheaders["Disposition-Notification-To"] = $this->xheaders["Reply-To"];
      else
         $this->xheaders["Disposition-Notification-To"] = $this->xheaders['From'];
   }

   if( $this->charset != "" ) {
      $this->xheaders["Mime-Version"] = "1.0";
      $this->xheaders["Content-Transfer-Encoding"] = $this->ctencoding;
   }

   $this->xheaders["X-Mailer"] = "Php/libMailv1.8";
   // include attached files
   if( count( $this->aattach ) > 0 ) {
      $this->_build_attachement();
   } else {
      $this->fullBody = $this->body;
   }

   reset($this->xheaders);
   while( list( $hdr,$value ) = each( $this->xheaders )  ) {
      if( $hdr != "Subject" )
         $this->headers .= "$hdr: $value\n";
   }


}

} // class Mail


?>
