<?php
/*!
      A user's address book
      AddressBook is a collection of Contact objects
*/

include_once( "./lib/webo/Contact.php" );

class AddressBook
{
/* default sortby field */
var $sortField = "name";
/* sort descending ? */
var $sortDescending = false;
var $owner = NULL;
var $enableFavorites = false;

function AddressBook( $user )
{
   if( $user == NULL )
            die( "AddressBook.AddressBook : owner undefined" );

   $this->favorites = array();
   $this->owner = $user->userid;
   $eowner = db_escape($this->owner);
   $group_oids = $user->getGroups();
   if( count($group_oids) > 0 ) {
      $gidlist = implode( "," , $group_oids );
      $this->ACIsql = " ( owner='$eowner' OR ( group_oid IN ($gidlist) ) )";
   } else {
      $this->ACIsql = "( owner='$eowner' )";
   }
}

function addItem( $obj ) // TODO : owner checks/set
{
   Webo::log( "AddressBook.addItem", LOG_DEBUG );
   if( ! $obj->get('oid') ) {
      $obj->set( "owner", $this->owner );
   }
   $obj->store();
   return true; // OK
}

function removeItem( $oid )
{
   $contact = $this->get( $oid, 'RWD' );
   if( $contact == NULL )
      return false;
   $ret = $contact->delete( $oid );
   if( $ret ) unset( $this->favorites[$oid] );
   return $ret;
}


function sortBy( $field, $descending = false )
{
   if( ! in_array( $field, array_keys($this->sortableFields())) )
      return false;
   $this->sortField = $field;
   $this->sortDescending = $descending;
   return true;
}

/*!
   private use only
*/
function _sqlOrderBy()
{
   if( empty($this->sortField) )
      return "";
   if( $this->sortDescending )
      return "ORDER BY $this->sortField DESC";
   else
      return "ORDER BY $this->sortField";
}

function getIndex( $field )
{
   return db_loadhashList( "SELECT DISTINCT $field,$field FROM contacts WHERE $this->ACIsql ORDER BY $field"  );
}

function getOidsForIndex( $indexName, $indexValue )
{
   return db_loadarray( "SELECT oid FROM contacts WHERE $this->ACIsql AND $indexName='$indexValue' " . $this->_sqlOrderBy() );
}

/*!
         return oids of ALL contacts (for the current user)
         @param maxrows limit the return to this number of oids
         @param orderby ordering field : for instance "organization,name"
*/
function getOids( $maxrows=NULL )
{
   $this->enableFavorites = false;
   $sql = "SELECT oid FROM contacts WHERE $this->ACIsql " . $this->_sqlOrderBy();
   if( $maxrows ) $sql .= " LIMIT $maxrows";
   Webo::log( "Abook.getOids SQL=$sql", LOG_DEBUG );
   return db_loadarray( $sql );
}

/*!
         get an item from the collection
         @param item oid
*/
function get( $oid, $access='R' )
{
   global $user;
   $obj = Contact::load( $oid );
   if( ! $obj ) {
      return NULL;
   }
   ////// access control
   // personal contact, access always allowed
   if( $obj->get('owner') == $this->owner ) {
      return $obj;
   }

   $gid = $obj->get("group_oid");
   if( $gid == NULL ){
      Webo::log( "AddressBook::get : no group_oid for object", LOG_DEBUG );
      return NULL;
   }

   if( in_array( $gid, $user->getGroups() ) ) {
      Webo::log( "Contact $oid access allowed for group $gid ", LOG_DEBUG );
      if( strpos( $obj->get('group_access'), $access ) !== false ) {
         return $obj;
      }
      return NULL;
   }
   else {
      Webo::log( "AddressBook::get access forbidden to contact $oid", LOG_DEBUG );
      return NULL;
   }
}


function search( $pattern, $fields, $operator )
{
   $operator = strtoupper($operator);
   if( $operator != "AND" && $operator != "OR" )
      $operator = "OR";
   $pattern = str_replace( "*", "%", $pattern );
   $sqlPattern = db_escape($pattern);

   $f = array_pop($fields);
   $sql = "$f LIKE '$sqlPattern'";
   foreach( $fields as $f ) {
      $sqlPattern = db_escape($pattern);
      $sql .= " $operator $f LIKE '$sqlPattern'";
   }
   return db_loadarray( "SELECT oid FROM contacts WHERE $this->ACIsql AND ( $sql ) " );
}

/*
         contact lookup
*/
function smartFind( $anyQuery, $resultArray )
{
   $this->enableFavorites = 1;
   // lookup for Webo users and personal contacts
   $precond = "( userid IS NOT NULL OR $this->ACIsql )";

   $q = trim( $anyQuery );
   $eq = db_escape($q);
   if( $q == "" )
            return 0;
   $oids = array();
   // lookup by initials
   if( strlen($q) <= 3 && strtoupper($q) == $q )
      $sql = "initials='$eq'";

   // lookup by email
   elseif( strpos( $q, "@" ) > 0 )
      $sql = "email1 LIKE '$eq' OR email2 LIKE '$eq' OR email3 LIKE '$eq' ";

   // lookup by full name
   elseif( strpos( $q, "," ) > 0 )
      $sql = "name LIKE '$eq'";

   // lookup by lastname or company
   else
      $sql = "lastname LIKE '$eq' OR organization LIKE '$eq'";

   // max 4 items per smart lookup
   $results = db_loadlist( "SELECT * FROM contacts WHERE $precond AND ( $sql )" , 4 );
   foreach( $results as $hash )
   {
      $this->addHistory($hash['oid'] );
      $resultArray[] = Contact::initialize($hash);
   }
   return count($resultArray);
}

/*!
   hook method used by global searches
   @param array a list of criterias to lookup
      each criteria is a hash containing these informations: field, value
   @param string the SQL operator to join query parts : OR, AND
*/
function executeQuery( $criterias, $globope )
{
   $sql = "";
   foreach( $criterias as $c )
   {
      $f = $c['field'];
      if( $f == "name" ) {
         $c['field'] = 'name';
      } elseif( $f == 'categories' ) {
         $c['field'] = $f;
      } else { // we skip unsupported criterias here
         next();
      }
      $sql .= Query::makeSQL( $c );
      $sql .= $globope;
   }
   $sql = substr( $sql, 0, strrpos($sql,$globope) );
   // no times for now in the result - to diffcult to handle (allday,multi-days event, formatting)
   $sql = "SELECT
            'contact' as objectclass, oid,
            'obj/vcard' as icon, name,
            phone1 as extra
            FROM contacts WHERE ( $this->ACIsql ) AND ( $sql ) ORDER BY name";
   $results = db_loadlist( $sql );
   return $results;
}

function getHistory( $itemcount = 10 )
{
   arsort( $this->favorites );
   $max= min( $itemcount, count($this->favorites));
   $i=0;
   foreach($this->favorites as $oid => $cnt ) {
         if( $i++ == $max )
            break;
         $ret[] = $oid;
   }
   return $ret;
}

/*!
   maintain an history of recently accessed items
   also maintain a list of favorites items
*/
function addHistory( $contact )
{
   if( $contact == NULL )
      return;
   @$this->favorites[$contact->get('oid')] ++;
   $ctcname = $contact->get('name');
   Webo::log( "Addressbook.addHistory( $ctcname )", LOG_DEBUG );
}


/**************************** STATIC REFERENTIAL METHODS **********************************/

/*!
         list of fields that can be used as the addressbook sort key
*/
function sortableFields()
{
   return array(
         '' => loc('Sort by'),
         'name' => loc('Name'),
         'name DESC' => loc('Name') . ' -',
         'organization' => loc('Company'),
         'organization DESC' => loc('Company') . ' -',
         'categories' => loc('Category'),
         'categories DESC' => loc('Category') . ' -',
         'created DESC' => loc('Creation date') . ' -',
         'modified DESC' => loc('Last modified') . ' -',
      );
}

/*!
   return an hash of Address book Smart filtering columns
   TODO "letter" filter will uses a specific handling see AddressBook class
*/
function listFilters()
 {
   return array(
        "" => "Filter By",
        "categories" => loc('Categories'),
        "organization" => loc('Companies'),
        "title" => loc('Job Title'),
        // TODO "letter" => "A-Z",
   );
}


function postalAddressToString( $adrHash )
{
   if( $adrHash == NULL )
      return "";
   extract( $adrHash );
   $str = sprintf( "%s\n%s %s\n%s %s", $street, $locality, $postalcode, $region, $country );
   return $str;
}

} // end  class
