<?php
/*
      Encapsulate an alarm : event reminder, due tasks , etc..

      Alarm status overview
      ---------------------------
      An user creates an alarm scheduled to run at 2003-03-10T11:55:00
      Alarm is created with a status = SCHEDULED
      if User delete the alarm (or delete related object, which has same effect), Alarm status is DELETED
      the 10 march at 11:55, the alarm daemon runs this alarm
      Alarm status is RUNNING
      if User snooze Alarm, status is set to SNOOZED
      if User close Alarm, status is CLOSED

      for now, waitime is set to 900

      $Id: Alarm.php,v 1.9 2003/04/23 00:13:38 lwest Exp $

      HISTORY
      $Log: Alarm.php,v $
      Revision 1.9  2003/04/23 00:13:38  lwest
      removed debug() calls

      Revision 1.8  2003/04/21 20:12:24  lwest
      clone method beta

      Revision 1.7  2003/04/14 19:39:35  lwest
      misc. bug fixes

      Revision 1.6  2003/04/06 10:24:54  lwest
      removed owner param in event::load

      Revision 1.5  2003/04/02 13:14:38  lwest
      *** empty log message ***

      Revision 1.4  2003/03/24 01:21:45  lwest
      *** empty log message ***

      Revision 1.3  2003/03/20 01:21:43  lwest
      added toHTML, fixed listAlarmsForUser to return a objects list

      Revision 1.2  2003/03/18 01:23:11  lwest
      *** empty log message ***


*/

class Alarm
{
   var $relatedobjectclass = NULL;
   var $relatedobjectoid = NULL;
   var $summary = NULL;
   var $owner = NULL;
   var $tsstart = NULL;
   var $tsend = NULL;
   var $tstrigger = NULL;
   var $timeoffset = 0;
   var $waittime = 900; /// 5 minutes delay between successive reminder
   var $status = 'SCHEDULED'; // scheduled by default
   var $interval = -1; // repeat interval time in seconds, -1 == no repetition,

   function Alarm()
   {
   }

   function load( $oid )
   {
      $eoid = db_escape( $oid );
      $sql = "SELECT * FROM alarms WHERE oid=$eoid";
      $obj = new Alarm();
      db_loadobject( $sql, &$obj );
      return $obj;
   }

   function getByRelatedObject( $objectclass, $objectoid )
   {
      if( $objectoid == NULL )
         return NULL;
      $oc= db_escape( $objectclass );
      $eoid = db_escape( $objectoid );
      $sql = "SELECT oid FROM alarms WHERE relatedobjectclass='$oc' AND relatedobjectoid=$eoid";
      webo::log( "Alarm::getByRelatedObject => " . $sql );
      if( db_loadhash( $sql, &$a ) )
         return Alarm::load( $a['oid'] );
      else
         return NULL;
   }

   /**
      * return a list of Alarms for a given user
      * @return array of hash, each containing alarm informations
      * @static
      **/
   function listAlarmsForUser( $userid, $t1, $t2 )
   {
      $uid = db_escape($userid);
      $template = new Alarm();
      $dt1 = db_datetime($t1);
      $dt2 = db_datetime($t2);
      $sql = "SELECT oid FROM alarms WHERE owner='$uid' AND dtstart <'$dt2' AND dtend > ' $dt1' AND ( status='SCHEDULED' OR status='SNOOZED' )";
      $alarms = db_loadobjectlist( $sql, &$template );
      return $alarms;
   }

   /*!
      list alarm which should be activated
   @static
   */
   function listActiveAlarms()
   {
      $alarms = array();
      $alarmToRun = $alarmToClose = array();
      $alarms = db_loadlist( "SELECT * FROM alarms WHERE status='SCHEDULED' OR status='SNOOZED' " );
      foreach( $alarms as $alm ) {
               if( $alm['dttrigger'] > $alm['dtend'] )
                     $alarmToClose[] = $alm['oid'];
               else
                     $alarmToRun[] = $alm['oid'];
      }
      $list = implode( ',', $alarmToClose );
      db_query( "UPDATE alarms SET status='AUTOCLOSED' WHERE oid IN( $list )" );
      return $alarmToRun;
   }


   function addEventReminder( $event, $timeoffset )
   {
      Webo::log( "addEventReminder( $event, $timeoffset )", LOG_DEBUG );
      // we have a User instance, "pseudo-cast" it into a  contact instance
      if( get_class($event) != 'event' ) {
               Webo::log("Alarm::addEventReminder : bad class", LOG_NOTICE );
               return NULL;
      }
      $this->relatedobjectclass = 'event';
      $relobjoid = $event->get('oid');
      if( ! $relobjoid ) {
               Webo::log("Alarm::addEventReminder : missing related oid", LOG_NOTICE);
               return NULL;
      }
      $this->relatedobjectoid = $relobjoid;
      $this->summary = "Event " . $event->get('summary');
      $this->owner = $event->get('owner');
      $this->tstrigger = $this->tsstart = $event->get('tsstart') + $timeoffset;
      $this->tsend = $event->get('tsend');
      $this->timeoffset = $timeoffset;
      $this->status = 'ACTIVE';
      return $this->store();
   }

   /**
    * return a user friendly text about the timeoffset
    * @static
    */
   function toHTML()
   {
      $timeoffset = $this->timeoffset;
      if( $timeoffset < 0 ) {
         $fmt = "<b>%s</b> in %s";
         $ofs = -$timeoffset;
      } else {
         $fmt = "<b>%s</b> <font color=red>%s ago</font>";
         $ofs = $timeoffset;
      }

      if( $ofs < 3600 )
         $time = round( $ofs / 60) . " minutes";
      elseif( $ofs <  86400 )
         $time = round($ofs/3600) . "h and " . round( $ofs / 60) . " minutes";
      $lbl = sprintf( $fmt, $this->summary, $time );
      return $lbl;
   }

   function snooze()
   {
      $this->setStatus( 'SNOOZED' );
      $this->setTriggerTime( $this->tsstart + $this->waitime );
   }

   function close()
   {
      $this->setStatus( 'CLOSED' );
   }

   function setStatus( $status )
   {
      $this->status = $status;
      $sth = db_prepare( "UPDATE alarms SET status=? WHERE oid=? " );
      db_execute( $sth, $this->status, $this->oid );
   }

   function setTriggerTime( $tstrigger )
   {
      $this->tstrigger = $tstrigger;
      $sth = db_prepare( "UPDATE alarms SET dttrigger=? WHERE oid=? " );
      db_execute( $sth, db_datetime($this->tstrigger), $this->oid );
   }

   function store()
   {
      $item = array(
         'summary' => $this->summary,
         'status' => $this->status,
         'dtstart' => db_datetime( $this->tsstart ),
         'dtend' => db_datetime( $this->tsend ),
         'dttrigger' => db_datetime( $this->tstrigger ),
         'relatedobjectoid' => $this->relatedobjectoid,
         'relatedobjectclass' => $this->relatedobjectclass,
         'owner' => $this->owner,
         'timeoffset' => $this->timeoffset,
         'waittime' => $this->waittime,
      );
      // update
      if( isset( $this->oid) ) {
         $ret = db_updatehash('alarms', $item, "oid" );
      }
      // insert
      else {
         $ret = db_inserthash('alarms', &$item, "oid" );
         $this->oid = $item['oid'];
      }
      return $ret;
   }

   function icon()
   {
      return "obj/bell";
   }

}

?>
