<?php

class Attendees
{

   var $event_oid = NULL;

   function Attendees()
   {
            $this->attendees = array();
   }

   function load( $event_oid )
   {
            $this->event_oid = $event_oid;
            $eoid = db_escape( $event_oid );
            $sql = "SELECT * FROM attendees WHERE event_oid=$eoid";
            $hash = db_loadlist( $sql );
            foreach( $hash as $atd ) {
                     $atd['icon'] = $this->calcIcon($atd);
                     $this->attendees[] = $atd;
            }
   }

   function getList()
   {
      return $this->attendees;
   }

   function setAttendeeStatus( $event_oid, $person_oid, $status )
   {
      $atd = array();
      $eoid = db_escape($event_oid);
      $poid = db_escape($person_oid);
      $ret = db_loadhash( "SELECT * FROM attendees WHERE event_oid=$eoid AND person_oid=$poid" , &$atd );
      if( ! is_array($atd))
               return false;
      if( $atd['status'] == $status )
               return;
      $sth = db_prepare( "UPDATE attendees SET status=? WHERE oid=?");
      db_execute( $sth, $status, $atd['oid'] );
   }

   /*!
      return true if user can set attendee status
      @static
   */
   function replyAllowed( $attendee, $user )
   {
      if( $user->getOid() == $attendee['person_oid'] )
         return true;
      else
         return false;
   }

   function setAll( $ctcOids, $roles, $status )
   {
      global $abook;
      // no addressbook => no way to invite anyone
      if( $abook == NULL )
         return;
      foreach( $ctcOids as $idx => $contact_oid )
      {
         // $ctc = $abook->get( $contact_oid );
         // because owner may not be accessible through Abook
         $ctc = Contact::load( $contact_oid );
         if( $ctc != NULL )
            $this->add( $ctc, $roles[$idx] );
      }
   }


   function add( $ContactOrUser, $role='ATTENDEE', $status='NEEDS-ACTION' )
   {
      // we have a User instance, "pseudo-cast" it into a  contact instance
      if( get_class($ContactOrUser) == 'user' ) {
         $ctc = $ContactOrUser->asContact();
         $atdtype = 'USER';
      // this is  a Contact instance
      } elseif(get_class($ContactOrUser) == 'contact' ) {
         $ctc = $ContactOrUser;
         $atdtype = 'CONTACT';
      } else {
         Webo::log( "bad class in Attendees::add" , LOG_ERROR );
         return false;
      }
      // check that contact is not allready among attendees
      foreach( $this->attendees as $idx => $atd ) {
         if( $atd['person_oid'] == $ctc->oid ) {
            unset( $this->attendees [$idx] );
            break;
         }
      }

      $atd = array (
         'event_oid' => $this->event_oid,
         'type' => $atdtype,
         'status' => $status,
         'person_oid' => $ctc->oid,
         'name'=> $ctc->get('name'),
         'email' => $ctc->get('email'),
         'role' => $role
      );
      $atd['icon'] = $this->calcIcon($atd);
      $this->attendees[] = $atd;
   }

   function clone()
   {
      for( $i=0; $i<count($this->attendees); $i++ ) {
         $this->attendees[$i]['status'] = 'NEEDS-ACTION';
         unset($this->attendees[$i]['oid']);
      }
   }

   function store()
   {
      intval($this->event_oid) or die( "Error : event_oid not set in Attendees.store() " );
      foreach( $this->attendees as $atd ) {
               $atd['event_oid'] = $this->event_oid;
               // icon is a transient prop
               unset( $atd['icon'] );

               if( isset($atd['oid']) ) {
                     $ret = db_updatehash( "attendees", $atd, "oid" );
               } else {
                     $ret = db_inserthash( "attendees", $atd, "oid" );
               }
               if( ! $ret )
                     Webo::log( "Attendees.store : failed to add $atd[name] ", LOG_WARNING );
      }
   }

   function updateEventStatus()
   {
      $evt = Event::load( $this->event_oid );
      // event is cancelled or allready accepted
      if( $evt->get('status') != 'TENTATIVE'  )
         return;

      foreach( $this->attendees as $atd ) {
         if( $atd['status'] != 'ACCEPTED' && $atd['status'] != 'DELEGATED' ) {
               return;
         }
      }
      $evt->setStatus( 'CONFIRMED' );
   }

   function calcIcon($atd)
   {
      switch( $atd['status'] )
      {
      case 'DECLINED':
               return "obj/attendee-declined";

      case 'ACCEPTED':
               return "obj/attendee-accepted";

      case 'DELEGATED':
               return "obj/attendee-needsaction";

      default:
               return "obj/attendee-needsaction";
      }
   }

   function listStatus()
   {
      return array(
               'NEEDS-ACTION' => 'Unreplied',
               'DECLINED' => 'Declined',
               'ACCEPTED' => 'Accepted',
         // NIY  'DELEGATED' => 'Delegated',
      );
   }

   /*!
      roles referential
      return all roles as a hash "role" => "role label"
      @static
   */
   function listRoles()
   {
      return array(
            'ATENDEE' => 'Invitee',
            'ORGANIZER' => 'Organizer',
            'NON-PARTICIPANT' => 'CC:',
      );
   }

   function getOrganizer()
   {
            foreach( $this->attendees as $idx => $atd ) {
                     if( $atd['role'] == 'ORGANIZER' )
                           return $atd;
            }
            return NULL;
   }


   /*!
            send an email alert to attendees
            @created 11 Feb 2003
            @param string the email text
   */
   function mailNotice( $mailBodyTemplate )
   {
      include_once( "./lib/mail/mail.php" );

      $evt = Event::load($this->event_oid);
      $organizer = $this->getOrganizer();

       // these lines are the *real* template solution : simple, fast, 100% php based
      ob_start();
      include_once( "./dev/event.mail.tpl" );
      $buffer = ob_get_contents();
      ob_end_clean();

      // print $buffer; exit();
      $m = new Mail();
      $m->Subject( "[WEBO Calendar] " . $evt->get('summary') );
      $m->From( Mail::prettyMailAddress( $organizer['email'], $organizer['name']) );
      $m->HTMLBody( $buffer );

      foreach( $this->attendees as $atd )
      {
         if( ! $atd['email'] ) {
               continue;
         }
         Webo::log( "Attendees.mailNotice : adding To:$atd[email]", LOG_NOTICE );
         $m->To( Mail::prettyMailAddress( $atd['email'], $atd['name'] ) );
      }
      // we send the mail only if necessary
      if( count($m->sendto) > 0 ) {
            Webo::log( "Attendees.mailNotice : sending email", LOG_NOTICE );
            $m->send();
      }

   }

}


?>
