<?php
/*!
Calendar represents a user calendar
it is a container for Events objects (meetings)
it provides  accessor, adding and  removing methods, as well as various selectors :

     $Id: Calendar.php,v 1.17 2003/05/14 09:06:58 lwest Exp $

*/

include_once( "./lib/webo/Event.php" );

class Calendar
{
   var $owner = NULL; // owner's login (aka user->userid)
   var $contact_oid = NULL; // owner's person_oid (aka user->contact_oid)
   var $holidays_year = NULL; // internal use for year holidays hash caching mechanic

   function Calendar( $user )
   {
      $this->owner = $user->userid;
      $this->contact_oid = $user->contact_oid;
      $euid = db_escape($this->owner);
      $this->ACIsql = "( attendees.event_oid=events.oid ) AND ( attendees.person_oid=$this->contact_oid )";
   }

   function removeItem( $key )
   {
      return Event::delete($key);
   }

   /*!
            add an event in the Calendar
            @TODO : AxsCtl
   */
   function addItem( &$obj )
   {
      if( ! @$obj->oid ) {
         $obj->set( "owner", $this->owner );
      }
      return $obj->store();
   }

   /*!
            load and instanciate an event from the database
            access permissions are checked. if access is denied, return NULL
            @param event oid
            @param type of action planned : R=read,W=write,D=delete
            @return an Event object
            @TODO : AxsCtl
   */
   function get( $oid, $accessType ='W' )
   {
      $obj = Event::load( $oid );

      if( ! $obj )
         Webo::log( "Event $oid not found", LOG_WARNING );
      return $obj;
   }

   function listNoteOids()
   {
      $eowner = db_escape($this->owner);
      $sql = "SELECT oid FROM notes WHERE owner='$eowner' ORDER BY modified,created DESC";
      return db_loadarray( $sql );
   }

   /*!
            get a Note from the database
            @param note object ID
   */
   function getNote( $oid )
   {
      $obj = Note::load( $this->owner, $oid );
      if( ! $obj )
         Webo::log("Note $oid not found",  LOG_NOTICE );
      return $obj;
   }

   /*!
            list events for a given date
   */
   function getEventsForDay( $objDate )
   {
      $objDate->setTime(0,0,0);
      $ts1 = db_datetime($objDate->getTimeStamp());
      $objDate->addDays(1);
      $ts2 = db_datetime($objDate->getTimeStamp());
      $owner = db_escape($this->owner);
      $owner = db_escape($this->owner);

      $sql = "SELECT events.oid FROM events, attendees
               WHERE $this->ACIsql AND dtstart<'$ts2' AND dtend>='$ts1'
               ORDER BY dtstart";
      Webo::log( $sql, LOG_DEBUG );
      return db_loadarray( $sql );
   }

   /*!
            list busy dates for a given month
            month is taken from the date parameter
            @param Date the date to get the month period from
            @return array of days with events in the form array( "YYYYMMDD", ... )
   */
   function getBusyDays( $objDate )
   {
      $objDate->setDay(1);
      $objDate->setTime(0,0,0);
      $ts1 = db_datetime($objDate->getTimeStamp());
      $objDate->addMonths(1);
      $ts2 = db_datetime($objDate->getTimeStamp());
      $sql = "SELECT DISTINCT DATE_FORMAT(dtstart,'%Y-%m-%d'), 1 FROM events, attendees
               WHERE $this->ACIsql AND dtstart<'$ts2' AND dtend>'$ts1'
               ORDER BY dtstart";
      return db_loadhashList( $sql );
   }

   function getHoliday( $date )
   {
      // we are using an internal cache this->holidays for perf. reasons
      if( empty($this->holidays) or $this->holidays_year != $date->getYear() ) {
         $this->holidays = $this->listHolidays( $date->getYear() );
         $this->holidays_year = $date->getYear();
      }
      $MMDD = $date->toString( "%m%d" );
      if( isset($this->holidays[$MMDD]))
         return $this->holidays[$MMDD];
      else
         return NULL;
   }


   function listHolidays( $year )
   {
      global $profile;
      webo::log( "Calendar::listHolidays($year)", LOG_DEBUG );
      if(! isset($profile['calendar_holidays']))
         return array();
      $ret = include_once("./locales/$profile[calendar_holidays]/holidays");
      // if needed, calculate the easter day and monday (this is why we want the year)
      if( function_exists("easter_date") && ( isset( $holidays['easter']) or isset( $holidays['eastermonday']) or isset( $holidays['pentecost']))) {
         $easterdat = new Date(easter_date($year));
         $noc = $easterdat->toString("%m%d");
         $holidays[$noc] = $holidays['easter'];
         if( isset( $holidays['eastermonday'])) {
            $dat = $easterdat;
            $dat->addDays(1);
            $noc = $dat->toString("%m%d");
            $holidays[$noc] = $holidays['eastermonday'];
         }
         // pentecote = paques + 50 (cf bible :)
         if( isset( $holidays['pentecost'])) {
            $dat = $easterdat;
            $dat->addDays(50);
            $noc = $dat->toString("%m%d");
            $holidays[$noc] = $holidays['pentecost'];
         }
        // asomption = paques + 30 (cf bible :)
         if( isset( $holidays['assomption'])) {
            $dat = $easterdat;
            $dat->addDays(39);
            $noc = $dat->toString("%m%d");
            $holidays[$noc] = $holidays['assomption'];
         }

      }
      return $holidays;
   }

   /*!
            list unreplied events for current user
            @return array of event oids
   */
   function listPendingEvents()
   {
      $eoid = db_escape($this->contact_oid);
      $sql = "SELECT event_oid FROM attendees WHERE status='NEEDS-ACTION' AND person_oid=$eoid";
      return db_loadarray( $sql );
   }

   /*!
   convert a date,time couple into a unix epoc
   @return int the timestamp or -1 if date is incorrect or -2 if time is incorrect
   @static
   @sample
   if( Calendar::DateToTS( "01/12/2002", "13:30", $ts ) < 0 )
      // there is a problem in the date or time

   */
   function DateToTS( $strdate, $strtime = NULL, $outTs= NULL )
   {
      // date processing
      $strdate = trim(strtolower($strdate));
      if( $strdate == "" ) {
         $outTs = NULL;
         return -1;
      }

      $date = new Date(); // aka today's date for defaulting
      $date->setTime(0,0,0);

      /************
         handle week date aka : sunday..saturday
      */
      $wday = array_search( $strdate, Date::listWeekdays());
      $offsets = Date::listCommonWords();
      $idx = array_search( $strdate, array_keys($offsets) );

      if( $wday !== false ) {
         $date->setWeekday( $wday );
      }

      /************
         handle common word aka : today,tomorrow...
      */
      elseif( $idx !== false ) {
         $date->addDays( $offsets[$strdate] );
      }

      /************
         handle normal date : dd, dd/mm , dd/mm/yyyy
      */
      else {

         $aD = split( "/", $strdate );
         // defaults
         if( count($aD) < 2 )
            $aD[1] = $date->getMonth();
         if( count($aD) < 3 )
            $aD[2] = $date->getYear();

         $date->setDate( $aD[2], $aD[1], $aD[0] );
      }
      // finally, date validity checking
      if( ! $date->isDateValid() )
         return -1;

      if( $strtime != NULL )
      {
         $aT = split( "[:\.hH]", trim($strtime) );
         isset($aT[1] ) or $aT[1] = "0";
         $date->setTime( $aT[0], $aT[1], 0 );

         if( ! $date->isTimeValid() )
            return -2;
      }
      // we now fill the out parameter
      $outTs = $date->getTimestamp();
      return $outTs;
   }

   /*! 
      hook method used by global searches 
      @param array a list of criterias to lookup
         each criteria is a hash containing these informations: field, value 
      @param string the SQL operator to join query parts : OR, AND 
   */
   function executeQuery( $criterias, $globope )
   {
      $sql = "";
      foreach( $criterias as $c )
      {
         $f = $c['field'];
         if( $f == "name" ) {
            $c['field'] = 'summary';
         } elseif( $f == 'date' ) {
            $c['field'] = 'dtstart';
            if( ($ts = Calendar::DateToTS( $c['value'], NULL)) > 0 )
               $c['value'] = db_datetime($ts);
         } elseif( $f == 'categories' || $f == 'location' ) {
            $c['field'] = $f;
         } else { // we skip unsupported search fields here
            next();
         }
         $sql .= Query::makeSQL( $c );
         $sql .= $globope;
      }
      $sql = substr( $sql, 0, strrpos($sql,$globope) );
      // no times for now in the result - to diffcult to handle (allday,multi-days event, formatting)
      $sql = "SELECT
               'event' as objectclass, events.oid as oid,
               'obj/event' as icon, summary as name,
               DATE_FORMAT( dtstart, '%d/%m/%y' ) as extra
               FROM events, attendees
               WHERE ( $this->ACIsql ) AND ( $sql ) ORDER BY summary";
      $results = db_loadlist( $sql );
      return $results;
   }

   /*!
   event's and todos priorities referential
   @return hash list of possible priorities
   @static
   */
   function listPriorities()
   {
      return array(
               "1" => "Urgent",
               "5" => "Normal",
               "9" => "Low",
      );
   }

   /*!
   privacy switch referential return ahash list of possible protection flags
   @static
   */
   function listPrivacies()
   {
      return array(
               "private" => "Personal",
               "public" => "Normal",
      );
   }


}


?>
