<?php
/*

Event class

DESCRIPTION

   modelize an calendar event object

SYNOPSIS

$evt = Event::load( 13 );
echo $evt->get("Summary") , " at ", $evt->get("starttime") , " ends at ", $evt->get("endtime");

$evt->set( "summary", "New meeting summary" );
$evt->store();

TODO
  reschedule alarm when changing event date/time

INFORMATIONS
        author : Leo West <west_leo@removeme.yahoo.com>

$Id: Event.php,v 1.16 2003/05/06 22:08:20 lwest Exp $

*/

require_once( './lib/webo/Attendees.php' );
require_once( './lib/webo/Alarm.php' );

class Event
{

   function Event()
   {
   }

   function initialize()
   {
      $this->info = array();
      $this->set( "privacy", 'public' );
      $this->set( "allday", 0 );
      $this->set( "priority", 5 );
      $this->set( "sequence", 0 );
   }

   function delete( $oid )
   {
      $eoid = db_escape( $oid );
      $ret = db_query( "DELETE FROM attendees WHERE event_oid=$eoid" );
      $ret = db_query( "DELETE FROM events WHERE oid=$eoid" );
      return $ret;
   }

   /*!
      prepare this event to be cloned
   */
   function clone()
   {
      Webo::log( "Event.clone", LOG_DEBUG );
      $atds = $this->getAttendees();
      $atds->clone();
      $this->setAttendees( $atds);
      unset($this->oid);
      return $this->store();
   }

   function load( $oid )
   {
      empty($oid) && die( "missing oid in Event.load( oid ) " );
      $eoid = db_escape( $oid );
      $obj = new Event();
      $obj->oid  = $oid;
      db_loadhash( "SELECT *, UNIX_TIMESTAMP(dtstart) AS tsstart, UNIX_TIMESTAMP(dtend) AS tsend FROM events WHERE oid='$eoid'", &$info );
      $obj->info = $info;
      return $obj;
   }

   function store()
   {
      if( ! $this->info['priority'] ) $this->info['priority']= 5;

      if( !empty($this->oid) )
      {
         $this->info['oid'] = $this->oid;
         $this->info['sequence']  ++;
         $this->info['dtstart'] = db_datetime( $this->info['tsstart'] );
         $this->info['dtend'] = db_datetime( $this->info['tsend'] );
         $this->info['modified'] = db_datetime( time() );
         unset( $this->info['tsstart'] );
         unset( $this->info['tsend'] );
         db_updatehash( "events", $this->info, "oid" );
         return true;
      }

      // insert a new object
      $this->info['sequence'] = 0;
      $this->info['created'] = db_datetime( time() );
      $this->info['dtstart'] = db_datetime( $this->info['tsstart'] );
      $this->info['dtend'] = db_datetime( $this->info['tsend'] );
      unset( $this->info['tsstart'] );
      unset( $this->info['tsend'] );
      $ret = db_inserthash( "events", $this->info, "oid" );
      if( ! $ret )
         return false;

      $this->oid = $this->info['oid'];
      // the default behavior for a private event : add owner as attendee, this is required
      if( ! isset($this->attendees) )
      {
         global $user;
         $attendees = new Attendees();
         // user object, role in meeting, reply status
         $attendees->add( $user, 'ORGANIZER', 'ACCEPTED' );
         $this->setAttendees( $attendees);
      }
      $this->attendees->event_oid = $this->oid;
      $this->attendees->store();
      return true;
   }

   function getAttendees()
   {
      $attendees = new Attendees();
      if( isset($this->oid) ) {
         $attendees->load( $this->oid );
      }
      return $attendees;
   }

   function set( $property, $value )
   {
      $property = strtolower($property);
      $value = trim($value);
      if( $value == "" ) {
               $value = NULL;
      }

      switch( $property )
      {

      case "oid":
            $this->oid = $value;
            break;

      case "duration":
            if( $value < 0 )
                     break;
            if( isset($this->info['tsstart']) ) {
                     $this->info['tsend'] = $this->info['tsstart'] + $value;
            }
            break;

      default:
            $this->info[$property] = $value;
            break;
      }
   }

   function setAttendees( $attendees )
   {
            // safe checks
            if( ! is_object($attendees))
                     return;
            $this->attendees = $attendees;
   }

   function reschedule( $timeoffsetinsecs )
   {
      if( intval($timeoffsetinsecs) === false ) {
         return false;
      }
      $this->info['tsstart'] += $timeoffsetinsecs;
      $this->info['tsend'] += $timeoffsetinsecs;
      $this->store();
   }

   // TODO : localized date format
   function setDate( $startorend, $strdate, $strtime )
   {
      $oldtsstart = @$this->info['tsstart'];
      $oldtsend = @$this->info['tsend'];

      $strdate = trim( $strdate );

      // time processing
      if( $strtime == NULL && $this->info['allday'] == 1 ) {
         $strtime = "00:00";
      }
      $strtime = trim( $strtime );

      $ts = Calendar::DateToTS( $strdate, $strtime );

      if( $ts == -1 ) {
         $this->errorMsg = "$startorend date $strdate is invalid";
         return false;
      }
      // bad time
      elseif( $ts == -2 ) {
         $this->errorMsg = "$startorend time $strtime is invalid";
         return false;
      }

      if( $startorend == "end" )
         $this->info['tsend'] = $ts;
      else
         $this->info['tsstart'] = $ts;

      // TODO when history support : "changed time" if oid is set (modification mode) and ts differs
      /*if( ( $oldtsstart != $this->info['tsstart'] ) or ( $oldtsend != $this->info['tsend'] )) {
      }*/
      return true;
   }

   /*
            update the event status
            @param status one of 'CONFIRMED', 'CANCELLED', 'TENTATIVE'
   */
   function setStatus( $status )
   {
            $sth = db_prepare( "UPDATE events SET status=?, modified=? WHERE oid=?" );
            db_execute( $sth, $status, db_datetime(time()), $this->oid );
            $this->info['status'] = $status;
   }

   function get( $property )
   {
      $property = strtolower($property);
      switch( $property )
      {
      case "oid":
            return @$this->oid;
            break;

      case "startdate":
            return ( isset($this->info['tsstart']) ? Date::Format( "%d/%m/%Y", $this->info['tsstart'] ) : "" );
            break;

      case "enddate":
            return ( isset($this->info['tsend']) ? Date::Format( "%d/%m/%Y", $this->info['tsend'] ): "");
            break;

      case "starttime":
            if( ! isset($this->info['tsstart']))
               return "";
            return ( $this->info['allday'] ? "" : @strftime( "%H:%M", $this->info['tsstart']) );
            break;

      case "endtime":
            if( ! isset($this->info['tsend']))
               return "";
            return ( $this->info['allday'] ? "" : @strftime( "%H:%M", $this->info['tsend']) );
            break;

      // duration in seconds
      case "duration":
            return $this->info['tsend'] - $this->info['tsstart'];
            break;

      default:
            if( isset($this->info[$property]) )
                     return $this->info[$property];
            break;
      }
      return NULL;
   }

   function check()
   {
      if( !empty($this->errorMsg))
               return  $this->errorMsg;

      if( $this->info['summary'] == NULL )
               return "summary is required";

      if( $this->info['tsstart'] == NULL or $this->info['tsend'] == NULL )
               return "incorrect date and time";

      if( $this->info['tsstart'] > $this->info['tsend'] )
               return "incorrect duration";

      return "OK";
   }

   function hasNoTime()
   {
      if( @$this->info['allday'] )
         return true;
      else
         return false;
   }

   /*!
      return the alarm object when an alarm is attached to this event
      @return Alarm object or NULL
   */
   function getAlarm()
   {
      return Alarm::getByRelatedObject( 'event', $this->oid );
   }

   /*
    * return icon for this event
    */
   function icon()
   {
      return "obj/event";
   }

   /*
      * return a list of possible delay for reminder feature
      * @static
      */
   function listAlarmOffsets()
   {
   return array(
            "" => "(set a reminder)",
               -900 => "15 minutes",
            -1800 => "30 minutes",
            -3600 => "1 hour",
            -14400 => "4 hours",
            -86400 => "one day",
            -432000 => "5 days"
            );
   }

}
// end class Event


?>
