<?php
/*!
   represents a global search query and result
   a Query is a set of triplet( search fields , operator, value)
   it is transmitted to a container object (calendar,addressbook), that implements the query method and
        translate the query into its internal query representation, and execute a lookup (search request)
   on its contained items ( events, contacts...)
        your can find implementations in TaskList and Calendar containers
*/

class Query
{
   var $resultCount = 0;

   function Query()
   {
      $this->criterias = array();
      $this->targetsContainers = array();
      $this->ope_joint = 'and';
   }

   function addCriteria( $field, $value , $operator = '=' )
   {
      $this->criterias[] = array(
         'field' => $field,
         'value' => $value,
         'operator' => $operator
      );
   }

   function addTarget( &$container )
   {
      if( is_object($container) && method_exists( $container, "executeQuery")) {
         $this->targetsContainers[] = $container;
         return true;
      } else {
         webo::log( "Query::addContainer : container do not define executeQuery method", LOG_WARNING );
         return false;
      }
   }


   function execute()
   {
      $results = array();
      // for each container that support query search (Calendar,TaskList for now)
      foreach( $this->targetsContainers as $container )
      {
         $res = $container->executeQuery( $this->criterias, $this->ope_joint );
         if( is_array($res) ) {
            $results = array_merge( $results, $res );
         }
      }
      $this->resultCount = count($results);
      return $results;
   }

   function makeSQL( $criteria )
   {
      $field = db_escape( $criteria['field']);
      $val = db_escape( $criteria['value']);
      switch( $criteria['operator'] )
      {
      case '=':
         $sqlOp = '='; break;
      case '!=':
         $sqlOp = '<>'; break;
      case '*=':
         $val = str_replace( "*", "%", $val );
         $sqlOp = 'LIKE';
         $val = "%$val%";
         break;
      case '<':
         $sqlOp = '<'; break;
      case '>':
         $sqlOp = '>'; break;
      }
      $sql = sprintf( " ( %s %s '%s' ) ", $field,$sqlOp,$val );
      return $sql;
   }

}

?>
