<?php
/*!

 Tasks list component
   represents a List of todos for a user
   offers methods related to todos : list, query, sorting,filtering,etc..

   $Id: TaskList.php,v 1.17 2003/05/13 15:08:52 lwest Exp $

*/

include_once( "./lib/webo/Todo.php" );

class TaskList
{
   var $owner = NULL; // owner's login (aka user->userid)
   var $contact_oid = NULL; // owner's person_oid (aka user->contact_oid)
   var $sortField = "priority"; // sort by priority
   var $sortDescending = false; // unused / deprec $w->addFieldDescriptor( "O", "organization" );

   function TaskList( $user )
   {
      $this->owner = $user->userid;
      $this->contact_oid = $user->contact_oid;
      $euid = db_escape($this->owner);
      $this->ACIsql = "assignee='$euid'";
   }

   /*!
            add a todo in the user's task list
            @TODO : AxsCtl, implements possibility to assign todos to another user
   */
   function addItem( $obj )
   {
      if( ! isset($obj->oid) ) {
         $obj->set( "owner", $this->owner );
      }
      if( ! $obj->get( "assignee") ) {
         $obj->set( "assignee", $this->owner );
      }
      $obj->store();
      return true;
   }

   /*!
            load and instanciate a Todo from the database
            access permissions are checked. if access is denied, return NULL
            @param Todo oid
            @param type of permission required : R=read,W=write,D=delete
            @return an Todo object or NULL if the permissions are not sufficient
   */
   function get( $oid, $accessType ='W' )
   {
      $obj = Todo::load( $oid );
      if( ! $obj )
         Webo::log( "Todo $oid not found", LOG_NOTICE );
      // enforce access permission TODO : same for Read access
      if( $accessType == 'D' && $this->owner != $obj->get('owner') )
         return NULL;
      elseif( $accessType == 'W' && $this->owner != $obj->get('assignee') && $this->owner != $obj->get('owner'))
         return NULL;
      elseif( $accessType == 'R' && $this->owner != $obj->get('assignee') && $this->owner != $obj->get('owner')) {
         if( $obj->get('privacy') != 'PUBLIC' )
            return NULL;
      }
      Webo::log( "TaskList.get($oid) OK", LOG_DEBUG );
      return $obj;
   }

   /*!
            list all user's Todo oids
            @return array of todos oids
   */
   function listOids( $extraclause =NULL )
   {
      if( $extraclause )
         $sql = "SELECT oid FROM todos WHERE $extraclause AND $this->ACIsql " . $this->_sqlOrderBy();
      else
         $sql = "SELECT oid FROM todos WHERE $this->ACIsql " . $this->_sqlOrderBy();
      return db_loadarray( $sql );
   }

   /*!
            list todos for a given date
            icnludes :
            - todos with tsdue = this date
            - late todos
            @return array of todos oids
   */
   function getTodosForDay( $objDate, $maxitems=NULL)
   {
      $objDate->setTime(0,0,0);
      $ts1 = $objDate->getTimeStamp();
      $cond = "( tsdue <= $ts1 OR tsdue IS NULL) AND ( status = 'IN-PROCESS' OR status = 'NEEDS-ACTION' ) ";
      $sql = "SELECT oid FROM todos WHERE ($this->ACIsql) AND $cond " . $this->_sqlOrderBy();
      if( intval($maxitems) )
         $sql .= " LIMIT $maxitems";
      return db_loadarray( $sql );
   }
   
   /*!
      list todos that are due for this date exactly (exact=true) or this date and before
   */
   function getDueTodos( $objDate, $exact=false )
   {
      $objDate->setTime(23,59,59);
      if( $exact ) {
         $dstr = $objDate->toString( '%Y-%m-%d' );
         $cond = "( DATE_FORMAT( FROM_UNIXTIME(tsdue),'%Y-%m-%d' ) = '$dstr' )";
      } else {
         $ts1 = $objDate->getTimeStamp();
         $cond = "( tsdue <= $ts1 )";
      }
      $sql = "SELECT oid FROM todos WHERE ($this->ACIsql) AND (status='IN-PROCESS' OR status='NEEDS-ACTION' ) AND $cond " . $this->_sqlOrderBy();
      return db_loadarray( $sql );
   }

   function getTodos( $maxitems=NULL)
   {
      $sql = "SELECT oid FROM todos WHERE status <> 'COMPLETED' AND status <> 'CANCELLED' AND ($this->ACIsql) " . $this->_sqlOrderBy();
      if( intval($maxitems) )
         $sql .= " LIMIT $maxitems";
      return db_loadarray( $sql );
   }

   /*!
            list todos not achieved (late) for current user
            @return array of todos oids
   */
   function listPendingTodos()
   {
      $sql = "SELECT oid FROM todos WHERE ($this->ACIsql) AND (status='NEEDS-ACTION' OR status='IN-PROCESS') " . $this->_sqlOrderBy();
      return db_loadarray( $sql );
   }

   /*!
         list todos to be done  one a given date  (tsdue property)
         @param a Date object
         @return array of todos oids
         @seeAlso Date/Date
   */
   function listDueTodos( $date )
   {
      $due = $date->toString('%Y-%m-%d' );
      $sql = "SELECT oid FROM todos WHERE ($this->ACIsql) AND FROM_UNIXTIME(tsdue)='$due' AND (status='NEEDS-ACTION' OR status='IN-PROCESS') " . $this->_sqlOrderBy();
      return db_loadarray( $sql );
   }

   /*!
            implements the global search query
            unkonown fields (say, email) are safely ignored
            date is mapped on Todo duedate (tsdue)
            name is mapped on Todo summary
   */
   function executeQuery( $criterias, $globope )
   {
      $sql = "";
      foreach( $criterias as $c )
      {
         $f = $c['field'];
         if( $f == "name" ) {
            $c['field'] = 'summary';
         } elseif( $f == "date" ) {
            $c['field'] = 'tsdue';
            Calendar::DateToTS($c['value'],NULL, &$c['value'] );
         } elseif( $f == "notes" ) {
            $c['field'] = 'description';
         }
         elseif( $f == 'categories' )
               ;
         else
               next();

         $sql .= Query::makeSQL( $c );
         $sql .= $globope;
      }
      $sql = substr( $sql, 0, strrpos($sql,$globope) );

      $sql = "SELECT 'todo' as objectclass, oid,
               'obj/todo' as icon, summary as name,
               status as extra
               FROM todos
               WHERE ( $this->ACIsql ) AND ( $sql ) ORDER BY summary";
      $results = db_loadlist( $sql );
      return $results;
   }

   function getIndex( $field )
   {
      // FIXME @HACK for date-type field : the day part only is interesting
      if( $field == 'tsdue' OR $field == 'tsmodified' OR $field == 'tsstart' ) {
         $field = " DATE_FORMAT( FROM_UNIXTIME($field),'%d/%m/%Y') ";
      }
      return db_loadhashList( "SELECT DISTINCT $field,$field FROM todos WHERE $this->ACIsql ORDER BY $field"  );
   }

   function getOidsForIndex( $field, $value )
   {
      $f =db_escape($field);
      $v =db_escape($value);
      // FIXME @HACK for date-type field : the day part only is interesting
/*      if( $field == 'duedate' OR $field == 'modified' OR $field == 'startdate' ) {
         $f = " DATE_FORMAT($field,'%d/%m/%Y') ";
      }*/
      return db_loadarray( "SELECT oid FROM todos WHERE $this->ACIsql AND $f='$v' " . $this->_sqlOrderBy() );
   }

   function sortBy( $field, $descending = false )
   {
      if( ! in_array( $field, array_keys($this->sortableFields())) )
         return false;
      $this->sortField = $field;
      $this->sortDescending = $descending;
      return true;
   }

   /*!
      private use only
   */
   function _sqlOrderBy()
   {
      if( empty($this->sortField) )
         return "";
      if( $this->sortDescending )
         return "ORDER BY $this->sortField DESC";
      else
         return "ORDER BY $this->sortField";
   }

   function completedStats()
   {
      return $this->_statisticsByDay( "tsdone", "tsdone <> NULL" );
   }


   function _statisticsByDay( $dateField, $query = "1=1" )
   {
      $stats = array();
      $sql = "SELECT DATE_FORMAT(FROM_UNIXTIME(tsclosed),'%d/%m/%Y') as done, DATE_FORMAT(FROM_UNIXTIME(tsdue),'%d/%m/%Y') as due,
                     DATE_FORMAT(FROM_UNIXTIME(tscreated),'%d/%m/%Y') as created
                     FROM todos WHERE $this->ACIsql ORDER BY created DESC";

      $a = db_loadlist( $sql );
      foreach( $a as $itm ) {
         extract( $itm);
         empty($done) or @$stats[$done]['done'] ++;
         empty($due) or @$stats[$due]['due'] ++;
         empty($created) or @$stats[$created]['created'] ++;
      };
      return $stats;
   }


   /**************************** STATIC REFERENTIAL METHODS **********************************/

   /*!
            list of fields that can be used as a tasklist sort key
   */
   function sortableFields()
   {
      // @TODO use a - suffix instead of a DESC (security)
      return array(
               '' => loc('Sort by'),
               'priority' => loc('Priority'),
               'priority DESC' => loc('Priority'). '-',
               'summary' => loc('Summary'),
               'summary DESC' => loc('Summary'). '-',
               'tsdue' => loc('Due date'),
               'tsdue DESC' => loc('Due date') . ' -',
               'tsmodified DESC' => loc('Last modified') . ' -',
               'categories' => loc('Category'),
               'categories DESC' => loc('Category') . ' -',
               'percent' => loc('%'),
               'percent DESC' => loc('%') . '-',
               'tscreated DESC' => loc('Creation date') . ' -',
      );
   }

}


?>
