<?php
/*
   A Todo object, aka a task in the tasklist

   $Id: Todo.php,v 1.14 2003/05/12 15:07:18 lwest Exp $

*/

class Todo
{

   function Todo()
   {
      $this->info = array();
      $this->tsdue = NULL;
      $this->tsdone = NULL;
   }

   function initialize()
   {
      $this->set( "privacy", 'public' );
      $this->set( "priority", 5 );
      $this->set( "sequence", 0 );
      $this->set( "status",  'NEEDS-ACTION' );
   }

   function load( $oid )
   {
      empty($oid) && die( "missing oid in Todo.load( oid ) " );
      $eoid = db_escape( $oid );
      $obj = new Todo();
      $obj->oid  = $oid;
      db_loadhash( "SELECT * FROM todos WHERE oid='$eoid'", &$info );
      $obj->info = $info;
      return $obj;
   }

   function store()
   {

      if( @$this->oid )
      {
         $this->info['oid'] = $this->oid;
         $this->info['sequence'] = $this->info['sequence'] + 1;
         $this->info['tsmodified'] = time();
         db_updatehash( "todos", $this->info, "oid" );
      }
      else {
         $this->info['sequence'] = 0;
         $this->info['tscreated'] = time();
         db_inserthash( "todos", $this->info, "oid" );
         $this->oid = $this->info['oid'];
      }
   }

   function complete()
   {
      $this->set( "status", "COMPLETED" );
      $this->info['percent'] = 100;
      $this->info['tsclosed'] = time();
      $this->store();
      // TODO : trigger a "todo completed" alarm if so configured
   }

      function start()
   {
      $this->set( "status", "IN-PROCESS" );
      $this->info['tsstarted'] = time();
      $this->store();
      // TODO : trigger a "todo completed" alarm if so configured
   }

   function delete()
   {
      $this->info['tsclosed'] = time();
      $this->set( "status", "CANCELLED" );
      $this->store();
      // TODO : trigger a "todo completed" alarm if so configured
   }

   function check()
   {
      if( !empty($this->errorMsg))
         return  $this->errorMsg;
      if( $this->info['assignee'] == NULL )
         return "assignee is required";
      if( $this->info['summary'] == NULL )
         return "summary is required";
      if( $this->info['status'] == NULL )
         return "incorrect status";
      return "OK";
   }

   function set( $property, $value )
   {
      $property = strtolower($property);
      $value = trim($value);
      $value == "" && $value = NULL;

      switch( $property )
      {
      case "oid":
            $this->oid = $value;
            break;

      case "percent":
         if( $value < 0 || $value > 100 )
                  return false;
         // special values that trigger specific actions
         if($value == 0 ) {
            $this->set("status", "NEEDS-ACTION" );
         } elseif( $value == 100 ) {
            $this->complete();
         }elseif( $this->info['percent'] == 0 ) {
            $this->start();
         }
         $this->info['percent'] = $value;
         break;

      case "duration":
         if( $value == NULL ) {
            $this->info['tsduration'] = NULL;
         }
         elseif( preg_match( '/(\d+)\s*d/i', $value, $r)) {
            $this->info['tsduration'] = $r[1] * 86400; // expressed in days
         }
         elseif( preg_match( '/(\d+)\s*h/i', $value, $r)) {
            $this->info['tsduration'] = $r[1] * 3600; // expressed in hours
         }
         break;

      case "duedate":
         if( $value == NULL ) {
            $this->info['tsdue'] = NULL;
         }
         elseif( Calendar::DateToTS( $value,  NULL, &$this->info['tsdue'] ) == -1 ) {
            $this->errorMsg = "invalid due date : $value";
            webo::log( $this->errorMsg, LOG_NOTICE );
         }
         break;

      default:
         $this->info[$property] = $value;
         break;
      }
   }

   // get a property
   function get( $property, $type="" )
   {
      $property = strtolower($property);
      switch( $property )
      {
      case "oid":
            return $this->oid;
            break;

      case "uuid":
            return md5( $GLOBALS['HOSTNAME'] . "::WEBO/tasks/todo::" . $this->oid );
            break;

      case "duedate":
            return Date::Format( "%d/%m/%Y", $this->info['tsdue'] );
            break;

      case "closeddate":
            return Date::Format( "%d/%m/%Y", $this->info['tsclosed'] );
            break;

      case "starteddate":
      case "startdate":
            return Date::Format( "%d/%m/%Y", $this->info['tsstarted'] );
            break;

      case "duration":
            if( @$this->info['tsduration'] == NULL )
               return "";
            elseif($this->info['tsduration'] < 86400 )
               return round($this->info['tsduration'] / 3600) . " hours"; // in hours
            else
               return round($this->info['tsduration'] / 86400) . " days"; // in days
            break;

      case "created":
            return Date::Format( "%d/%m/%Y", $this->info['tscreated'] );
            break;

      case "modified":
         return Date::Format( "%d/%m/%Y", $this->info['tsmodified'] );
         break;

      default:
            if( isset($this->info[$property]) )
                     return $this->info[$property];
            break;
      }
      return NULL;
   }

   function isLate()
   {
      if( $this->info['status'] == 'COMPLETED' || $this->info['status'] == 'CANCELLED' )
         return false;
      else
         return ($this->tsdue != NULL && $this->tsdue < time() );
   }

   function isCompleted()
   {
      return $this->info['status'] == 'COMPLETED';
   }

   function icon()
   {
      return "obj/todo";
   }

   function statusIcon()
   {
      switch( $this->info['status'] ) {
      case 'COMPLETED':
         return "flags/completed.png";
      case 'IN-PROCESS':
         return "flags/in-process.png";
      case 'CANCELLED':
         return "flags/cancelled";
      default:
         return NULL; // TODO
      }
   }

   function listStatus()
   {
      return array( "NEEDS-ACTION",  "IN-PROCESS", "COMPLETED", "CANCELLED" );
   }

} // end class Todo


?>
