<?php
/**
   Webo user class

   To load a specific user, User::load($userid)
   To authenticate and load a user, use Authent( userid, password )
   To get extra contact informations (name,email,etc) use asContact()

   $Log: User.php,v $
   Revision 1.11  2003/05/09 21:10:46  lwest
   DB directory moved to db

   Revision 1.10  2003/04/21 20:12:24  lwest
   clone method beta

   Revision 1.9  2003/04/14 19:34:51  lwest
   roles support : added listRoles method

   Revision 1.8  2003/04/12 17:30:52  lwest
   replaced timer to webo::timer

   Revision 1.7  2003/04/08 03:23:39  lwest
   new functionalities

   Revision 1.6  2003/03/24 01:16:32  lwest
   added listUsers and listCollaborators methods


*/
class User
{
   var $userid;
   var $userpassword;
   var $name;
   var $email;
   var $status;
   var $contact_oid;

   function User()
   {
   }

   function getOid()
   {
      return $this->contact_oid;
   }


   function isAuthenticated()
   {
      return $this->status == 'enabled';
   }

   function isLocked()
   {
      return $this->status == 'locked';
   }

   /*!
      check user credentials
      @return User object if authent ok, NULL otherwise
   */
   function authent( $userid, $password )
   {
      $u = db_escape( $userid );
      $p = md5( $password );
      $row = array();
      db_loadhash( "SELECT userid FROM users WHERE userid='$u' AND userpassword='$p'", &$row );
      if( $row == NULL ) {
         $this->errmsg = "Invalid username or password";
         return 0;
      } else {
         $this = User::load( $row['userid'] );
         return $this->canLogin();
      }
   }

   /*!
      verify that this account status allow to log into Webo
      @return 1 if ok, 0 if not allowed $this->errmsg contains the reason
   */
   function canLogin()
   {
      switch( $this->status )
      {
      case 'enabled':
         return 1;
         break;
      case 'disabled':
            $this->errmsg = "Your account is disabled";
            break;
      case 'locked':
            $this->errmsg = "Account is locked by Webo administrator";
            break;
      case 'deleted':
            $this->errmsg = "Invalid username or password";
            break;
      }
      return 0;
   }

   /*!
      try an automatic login ar is taken from the autologin cookie
      @return 1 if ok, 0 if not allowed $this->errmsg contains the reason
   */
   function autoLogin( $loginkey )
   {
      $key = db_escape( $loginkey );
      $row = array();
      $ret = db_loadhash( "SELECT task_user FROM automatedtasks WHERE task_type='login' AND task_authkey='$key' ", &$row );
      if( $ret ) {
         $this = User::load( $row['task_user'] );
         return $this->canLogin();
      } else {
         setcookie( "webo_autologin", NULL, 0, "/" );
      }
      $this->errmsg= "Your autosignin key has expired. Please re-authentify";
      return 0;
   }


   function enableAutoLogin( $enabled )
   {
      Webo::log( "enableAutoLogin( $enabled )", LOG_DEBUG );
      $u = db_escape( $this->userid );
      if( $enabled ) {
         $key = md5( serialize( $this )); // dirty but simple
         $key = db_escape( $key );
         $timeout = ( 86400 * 5) + time();
         $sql = "INSERT INTO automatedtasks(task_user,task_type,task_authkey) VALUES ('$u', 'login', '$key') ";
         $ret1 = db_query( $sql );
         $ret = setcookie( "webo_autologin", $key, $timeout, "/" );
         Webo::log( "webo_autologin DB=>$ret1 cookie=>$ret", LOG_DEBUG );
         Webo::log( "autologin enabled $key", LOG_INFO );
      } else {
         $ret1 = db_query( "DELETE FROM automatedtasks WHERE task_user='$u' AND task_type='login' " );
         $ret = setcookie( "webo_autologin", NULL, 0, "/" );
         Webo::log( "autologin disabled", LOG_INFO );
      }
   }

   function load( $userid )
   {
      $u = db_escape( $userid );
      db_loadhash( "SELECT usr.userid, usr.userpassword, usr.status,
                     ctc.oid, name, firstname, lastname, email1 AS email
               FROM users usr, contacts ctc WHERE usr.userid=ctc.userid AND usr.userid='$u' ", &$row );
      $user = new User();
      $user->userid = $row['userid'];
      $user->userpassword = $row['userpassword'];
      $user->name = $row['name'];
      $user->email = $row['email'];
      $user->status = $row['status'];
      $user->contact_oid = $row['oid'];
      return $user;
   }

   /*!
            user membership  : groups that user is a member of
            @return array oids of groups
   */
   function getGroups()
   {
      $this->groups = NULL;
      if( $this->contact_oid == NULL ) {
         Webo::log( "user $this->userid has no groups", LOG_WARNING );
         $this->groups = array();
      }

      if( ! isset($this->groups) )
      {
         $oid = db_escape($this->contact_oid);
         $this->groups = db_loadarray("SELECT g.oid FROM groups g,groupmembers m
               WHERE g.oid=m.group_oid AND m.contact_oid='$this->contact_oid'"
         );
      }
      return $this->groups;
   }

   function listUsers()
   {
      $users = db_loadhashlist("SELECT u.userid,c.name FROM users u,contacts c
                  WHERE u.userid=c.userid
                  ORDER BY c.name");
      return $users;
   }

   /*!
   List user's coworkers
   return a hashlist of users in the same groups
   @return hash userid => commonname
   */
   function listCollaborators()
   {
      $grplist = join(",", $this->getGroups());
      $users = db_loadhashlist("
            SELECT u.userid,c.name FROM users u,contacts c, groupmembers g
            WHERE g.group_oid IN($grplist) AND u.userid=c.userid AND c.oid = g.contact_oid
            ORDER BY c.name" );
      return $users;
   }

   /*!
   List user roles
   return an array
   @return array list of roles granted to the user
   */
   function listRoles()
   {
      if( @is_array($this->roles))
         return $this->roles;
      $euid = db_escape($this->userid);
      $roles = db_loadarray("SELECT role FROM autpermission WHERE userid='$euid' ORDER BY role" );
      return $roles;
   }

   /*!
      set all user's role at once
      @param : an array of roles
   */
   function setRoles( $lstRoles)
   {
      if( ! is_array($lstRoles))
         return false;
      db_query("delete from autpermission WHERE userid='$euid'" );
      $sth = db_prepare( "insert into autpermission(userid,role) VALUES( '$euid', ? )" );
      foreach( $lstRoles as $role ) {
         db_execute( $sth, $role);
      }
   }

   function asContact()
   {
      if( is_object($this->contact))
         return $this->contact;
      else
         return Contact::load( $this->contact_oid );
   }

};


?>
