<?php
/*!

   A small URL caching utility
   add some support for storing some URL infos in the webcache database table :
    - page description (wbc_name)
    - content-type
    - owner of this item, NULL = public accessible page
    - next update datetime
    - last update datatime
    - page status : UNCACHED, INCACHE, DELETED, E_<http_error_code>
   
   $Date: 2003/05/18 07:48:08 $
   
   @depends file/cache.php
   @db table webcache

*/

include_once( "./lib/file/cache.php"); // @depends
include_once( "./lib/net/HTTP/client.php"); // @depends

class webo_cache extends file_cache
{
   /// default refresh interval for cached items (in seconds)
   var $refresh_interval = 14400; // 4 hours

   function webo_cache()
   {
      file_cache::file_cache( $GLOBALS['CACHE_DIR'] ); // instanciate the file cache (disk only)
   }

   function insertUrl( $url, $owner, $name = NULL )
   {
      if( ! parse_url($url)) {
         return NULL;
      }
      ( $name == NULL ) && $name = preg_replace( "|^\w+://|", "", $url );
      $hash = array(
         'wbc_url' => $url,
         'wbc_owner' => $owner,
         'wbc_name' => $name,
         'wbc_contenttype' => NULL,
         'wbc_status' => 'UNCACHED'
      );
      $ret = db_inserthash( 'webcache', $hash, 'wbc_id' );
      return $ret;
   }

   /*!
      list items for a given user (owner set) or only public items (NULL)
      do not return deleted items
   */
   function listOids( $owner = NULL  )
   {
      if( $owner ) {
         $eo = db_escape($owner);
         return db_loadarray( "SELECT wbc_id FROM webcache WHERE wbc_status <> 'DELETED' AND ( wbc_owner='$eo' OR wbc_owner IS NULL )" );   
      } else {
         return db_loadarray( "SELECT wbc_id FROM webcache WHERE wbc_status <> 'DELETED' AND wbc_owner IS NULL" );            
      }
   }

   function getItemsList( $owner )
   {
      $euid = db_escape( $owner );
      return db_loadhashlist( "SELECT wbc_id,wbc_name FROM webcache WHERE (wbc_owner='$euid' OR wbc_owner IS NULL) AND wbc_status <> 'DELETED' ORDER BY wbc_name" );
   }

   function removeUrl( $url )
   {
      $eurl = db_escape($url);
      $cnt = db_query( "DELETE FROM webcache WHERE wbc_url='$eurl'");
      return $cnt;
   }

   function get( $oid )
   {
      $sth = db_prepare( "SELECT * FROM webcache WHERE wbc_id=?" );
      $cur = db_execute( $sth, $oid );
      $hash = db_nextrow($cur);
      return $hash;
   }

   /* same as parent
   function getData( $url )
   {
      $h = db_load( "SELECT wbc_url FROM webcache WHERE wbc_id=$oid" );
      print_r($h);
      return $h;
   }
   */
   function updateCache( $updateAll = false, $verbose = false )
   {
      $this->verbose = $verbose;
      // update all items (except deleted)
      if( $updateAll )
         $sql = "SELECT * FROM webcache WHERE wbc_status <> 'DELETED'";
      // update only items that were never cached or in error
      else
         $sql = "SELECT * FROM webcache WHERE wbc_status = 'UNCACHED' OR wbc_status LIKE 'E_%' ";
      $sth = db_prepare( $sql );
      $cur = db_execute( $sth );
      $DT = strftime( "%Y-%m-%d %H:%M:%S", time());
      if( $this->verbose ) echo "$DT - updating Webo webcache\n";
      while( $hash = db_nextrow($cur))
      {
         $this->updateCacheFile( $hash );
      }

      $sql = "SELECT * FROM webcache
                  WHERE wbc_nextupdate<?
                  AND wbc_status <> 'DELETED'
                  ORDER BY wbc_lastupdated DESC";
      $tsnow = time();
      $sth = db_prepare( $sql );
      $cur = db_execute( $sth, $tsnow );

      while( $hash = db_nextrow($cur))
      {
         $this->updateCacheFile( $hash );
      }
      if( $this->verbose ) echo "$DT update ended\n";
   }

   function updateItem( $oid )
   {
      $sth = db_prepare( "SELECT * FROM webcache WHERE wbc_id=?" );
      $cur = db_execute( $sth, $oid );
      $hash = db_nextrow($cur);
      if( $hash )
         return $this->updateCacheFile( $hash );
      else
         return NULL;
   }

   function updateCacheFile( $hash )
   {
      if( ! @is_object( $this->http )) {
         $this->http = new  Net_HTTP_Client();
         $this->http->setProtocolVersion( "1.1" );
         //$this->http->setDebug( DBGOUTDATA + DBGINDATA );
      }

      // echo "updateCacheFile( $hash )"; print_r($hash);
      if( $this->verbose ) echo "updating cache for $hash[wbc_url] ...";

      $reply = $this->http->get( $hash['wbc_url'] );
      // print_r($this->http);
      if( $reply != 200 )
      {
         $hash['wbc_status'] = 'E_' . $reply;
         // shall we increment the nextupdate stuff ??
         db_updatehash( "webcache", $hash, 'wbc_id' );
         if( $this->verbose ) echo "ERR $reply\n";
         return false;
      }
      else {
         $content = $this->http->getBody();
         // store the file on the disk cache
         $cachefile = $this->put( $hash['wbc_url'], $content );
         $hash['wbc_contenttype'] = $this->http->getHeader('Content-Type');
         $hash['wbc_lastupdated'] = time();
         $hash['wbc_nextupdate'] = time() + $this->refresh_interval;
         $hash['wbc_status'] = 'IN-CACHE';
         $hash['wbc_cachefile'] = $cachefile;
         $ret = db_updatehash( "webcache", $hash, 'wbc_id' );
         if( $this->verbose ) echo "OK\n";
         return $ret;
      }
   } // updateCacheFile

} // class webo_cache




?>
